package com.usky.dm.controller.web;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.usky.common.core.bean.ApiResult;
import com.usky.common.core.bean.CommonPage;
import com.usky.dm.domain.BaCwInfo;
import com.usky.dm.domain.WorkOrder;
import com.usky.dm.service.BaCwInfoService;
import com.usky.dm.service.vo.BaCwCollectVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

/**
 * ba 厕位
 * @author ya
 * @since 2022-05-25
 */
@RestController
@RequestMapping("/baCwInfo")
public class BaCwInfoController {


    @Autowired
    private BaCwInfoService baCwInfoService;
//
//
//
//    @GetMapping("addByConfig")
//    public ApiResult<Void> addByConfig(){
//        baCwInfoService.addByConfig();
//        return ApiResult.success();
//    }

    /**
     * 分页
     * @param current 页数
     * @param size 条数
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param status 状态
     * @param floor 楼层
     * @param type 类型(male男厕，female女厕)
     * @param isAlert true故障false正常
     * @param name 名称
     * @return
     */
    @GetMapping("/page")
    public ApiResult<CommonPage<BaCwInfo>> page(@RequestParam Integer current,
                                                @RequestParam Integer size,
                                                @RequestParam(required = false) Date startTime,
                                                @RequestParam(required = false) Date endTime,
                                                @RequestParam(required = false) String status,
                                                @RequestParam(required = false) String floor,
                                                @RequestParam(required = false) String type,
                                                @RequestParam(required = false) Boolean isAlert,
                                                @RequestParam(required = false) String name){
        return ApiResult.success(baCwInfoService.list(current, size, startTime, endTime,status,floor,type,isAlert,name));
    }

    /**
     * 查看全部厕位
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param status 状态
     * @param floor 楼层
     * @param type 类型(male男厕，female女厕)
     * @return
     */
    @GetMapping("/all")
    public ApiResult<List<BaCwInfo>> all(@RequestParam(required = false) String status,
                                         @RequestParam(required = false) String floor,
                                         @RequestParam(required = false) String type,
                                         @RequestParam(required = false) Date startTime,
                                         @RequestParam(required = false) Date endTime){
        LambdaQueryWrapper<BaCwInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper
                .between(null != startTime && null != endTime,BaCwInfo::getCreateTime,startTime,endTime)
                .eq(StringUtils.isNotBlank(status),BaCwInfo::getStatus,status)
                .eq(StringUtils.isNotBlank(floor),BaCwInfo::getFloor,floor)
                .eq(StringUtils.isNotBlank(type),BaCwInfo::getWcType,type)
                .orderByDesc(BaCwInfo::getId);
        return ApiResult.success(baCwInfoService.list(queryWrapper));
    }

    /**
     * 添加厕位告警
     * @param workOrder
     * @return
     */
    @PostMapping("/addAlert")
    public ApiResult<Void> addAlert(@RequestBody WorkOrder workOrder){
        baCwInfoService.addAlert(workOrder);
        return ApiResult.success();
    }

    /**
     * 处理告警
     * @param workOrder
     * @return
     */
    @PutMapping("/checkAlert")
    public ApiResult<Void> checkAlert(@RequestBody WorkOrder workOrder){
        baCwInfoService.checkAlert(workOrder);
        return ApiResult.success();
    }

    /**
     * 厕位状态统计
     * @param floor 楼层
     * @param type 类型
     * @return
     */
    @GetMapping("/collect")
    public ApiResult<BaCwCollectVO> collect(@RequestParam(required = false) String floor,
                                            @RequestParam(required = false) String type){
        return ApiResult.success(baCwInfoService.collect(floor,type));
    }


    /**
     * 厕位状态楼层统计
     * @param floor 楼层
     * @param type 类型
     * @return
     */
    @GetMapping("/collectByFloor")
    public ApiResult<List<BaCwCollectVO>> collectByFloor(@RequestParam(required = false) String floor,
                                            @RequestParam(required = false) String type){
        return ApiResult.success(baCwInfoService.collectByFloor(floor,type));
    }
}

