package com.usky.dm.controller.web;


import com.usky.common.core.bean.ApiResult;
import com.usky.common.core.bean.CommonPage;
import com.usky.dm.domain.CommunicationApi;
import com.usky.dm.service.CommunicationApiService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * 通信接口
 *
 * @author ya
 * @since 2022-02-22
 */
@RestController
@RequestMapping("/communicationApi")
public class CommunicationApiController {


    @Autowired
    private CommunicationApiService communicationApiService;


    /**
     * 新增记录
     * @param communicationApi
     * @return
     */
    @PostMapping
    public ApiResult<Void> add(@RequestBody CommunicationApi communicationApi){
        communicationApiService.add(communicationApi);
        return ApiResult.success();
    }

    /**
     * 修改记录
     */
    @PutMapping
    public ApiResult<Void> edit(@RequestBody CommunicationApi communicationApi)
    {
        communicationApiService.update(communicationApi);
        return ApiResult.success();
    }

    /**
     * 记录查询
     * @param current 页数
     * @param size 条数
     * @param storeName 厂商名称
     * @param transferType 传输方式
     * @return
     */
    @GetMapping("/list")
    public ApiResult<CommonPage<CommunicationApi>> list(@RequestParam Integer current,
                                                        @RequestParam Integer size,
                                                        @RequestParam(required = false) String storeName,
                                                        @RequestParam(required = false) String transferType)
    {
        return ApiResult.success(communicationApiService.list(current,size,storeName,transferType));
    }


    /**
     * 导出记录
     * @param storeName 厂商名称
     * @param transferType 传输方式
     * @return
     */
    @GetMapping("/export")
    public ApiResult<String> export(@RequestParam(required = false) String storeName,
                                    @RequestParam(required = false) String transferType){
        return ApiResult.success(communicationApiService.export(storeName,transferType));
    }


    /**
     * 删除记录
     */
    @DeleteMapping("/{id}")
    public ApiResult<Void> remove(@PathVariable("id") Integer id)
    {
        communicationApiService.remove(id);
        return ApiResult.success();
    }
}

