package com.usky.dm.controller.web;


import com.usky.common.core.bean.ApiResult;
import com.usky.common.core.bean.CommonPage;
import com.usky.dm.domain.Company;
import com.usky.dm.service.CompanyService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * 单位表
 *
 * @author ya
 * @since 2022-02-22
 */
@RestController
@RequestMapping("/company")
public class CompanyController {


    @Autowired
    private CompanyService companyService;


    /**
     * 记录查询
     * @param current 页数
     * @param size 条数
     * @param name 名称
     * @return
     */
    @GetMapping("/list")
    public ApiResult<CommonPage<Company>> list(@RequestParam Integer current,
                                               @RequestParam Integer size,
                                               @RequestParam(required = false) String name)
    {
        return ApiResult.success(companyService.list(current,size,name));
    }


    /**
     * 新增记录
     * @param company
     * @return
     */
    @PostMapping
    public ApiResult<Void> add(@RequestBody @Validated Company company){
        companyService.add(company);
        return ApiResult.success();
    }

    /**
     * 修改记录
     */
    @PutMapping
    public ApiResult<Void> edit(@RequestBody @Validated Company company)
    {
        companyService.update(company);
        return ApiResult.success();
    }

    /**
     * 删除记录
     */
    @DeleteMapping("/{id}")
    public ApiResult<Void> remove(@PathVariable("id") Integer id)
    {
        companyService.remove(id);
        return ApiResult.success();
    }
}

