package com.usky.dm.controller.web;


import com.usky.common.core.bean.ApiResult;
import com.usky.common.core.bean.CommonPage;
import com.usky.dm.domain.DeviceChannel;
import com.usky.dm.service.DeviceChannelService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 *  监测通道
 *
 * @author ya
 * @since 2022-02-22
 */
@RestController
@RequestMapping("/deviceChannel")
public class DeviceChannelController {

    @Autowired
    private DeviceChannelService deviceChannelService;



    /**
     * 新增记录
     * @param deviceChannel
     * @return
     */
    @PostMapping
    public ApiResult<Void> add(@RequestBody @Validated DeviceChannel deviceChannel){
        deviceChannelService.add(deviceChannel);
        return ApiResult.success();
    }

    /**
     * 修改记录
     */
    @PutMapping
    public ApiResult<Void> edit(@RequestBody @Validated DeviceChannel deviceChannel)
    {
        deviceChannelService.update(deviceChannel);
        return ApiResult.success();
    }

    /**
     * 记录查询
     * @param current 页数
     * @param size 条数
     * @param name 监测对象名称
     * @param dwtype 通道类型名称
     * @return
     */
    @GetMapping("/list")
    public ApiResult<CommonPage<DeviceChannel>> list(@RequestParam Integer current,
                                                     @RequestParam Integer size,
                                                     @RequestParam(required = false) String name,
                                                     @RequestParam(required = false) Integer dwtype)
    {
        return ApiResult.success(deviceChannelService.list(current,size,name,dwtype));
    }

    /**
     * 导出记录
     * @param name 名称
     * @param dwtype 通道名称
     * @return
     */
    @GetMapping("/export")
    public ApiResult<String> export(@RequestParam(required = false) String name,
                                    @RequestParam(required = false) Integer dwtype){
        return ApiResult.success(deviceChannelService.export(name,dwtype));
    }


    /**
     * 删除记录
     */
    @DeleteMapping("/{id}")
    public ApiResult<Void> remove(@PathVariable("id") Integer id)
    {
        deviceChannelService.remove(id);
        return ApiResult.success();
    }
}

