package com.usky.dm.controller.web;


import com.usky.common.core.bean.ApiResult;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.log.annotation.Log;
import com.usky.common.log.enums.BusinessType;
import com.usky.dm.domain.Device;
import com.usky.dm.mapper.vo.DeviceCollectVO;
import com.usky.dm.mapper.vo.DeviceRequestVO;
import com.usky.dm.service.DeviceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;


/**
 *  设备
 * @author ya
 * @since 2022-02-22
 */
@RestController
@RequestMapping("/device")
public class DeviceController {


    @Autowired
    private DeviceService deviceService;


    /**
     * 新增记录
     * @param device
     * @return
     */
    @Log(title = "新增设备", businessType = BusinessType.INSERT)
    @PostMapping
    public ApiResult<Void> add(@RequestBody @Validated Device device){
        deviceService.add(device);
        return ApiResult.success();
    }

    /**
     * 修改记录
     */
    @Log(title = "修改设备", businessType = BusinessType.UPDATE)
    @PutMapping
    public ApiResult<Void> edit(@RequestBody @Validated Device device)
    {
        deviceService.update(device);
        return ApiResult.success();
    }

    /**
     * 分页记录
     * @param deviceRequestVO
     * @return
     */
    @PostMapping("/list")
    public ApiResult<CommonPage<Device>> list(@RequestBody DeviceRequestVO deviceRequestVO)
    {
        return ApiResult.success(deviceService.list(deviceRequestVO));
    }

    /**
     * 导出记录
     * @param deviceRequestVO
     * @return
     */
    @PostMapping("/export")
    public ApiResult<String> export(@RequestBody DeviceRequestVO deviceRequestVO){
        return ApiResult.success(deviceService.export(deviceRequestVO));
    }

    /**
     * 设备统计
     * @param isLine 1是
     * @return
     */
    @GetMapping("/collect")
    public ApiResult<List<DeviceCollectVO>> getCollect(@RequestParam(required = false) Integer isLine){
        return ApiResult.success(deviceService.getCollect(isLine));
    }



    /**
     * 删除记录
     */
    @Log(title = "删除设备", businessType = BusinessType.DELETE)
    @DeleteMapping("/{id}")
    public ApiResult<Void> remove(@PathVariable("id") Integer id)
    {
        deviceService.remove(id);
        return ApiResult.success();
    }

}

