package com.usky.dm.controller.web;


import com.usky.common.core.bean.ApiResult;
import com.usky.common.core.bean.CommonPage;
import com.usky.dm.domain.DeviceType;
import com.usky.dm.service.DeviceTypeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 *  设备类型
 *
 * @author ya
 * @since 2022-02-22
 */
@RestController
@RequestMapping("/deviceType")
public class DeviceTypeController {


    @Autowired
    private DeviceTypeService deviceTypeService;


    /**
     * 新增记录
     * @param deviceType
     * @return
     */
    @PostMapping
    public ApiResult<Void> add(@RequestBody @Validated DeviceType deviceType){
        deviceTypeService.add(deviceType);
        return ApiResult.success();
    }

    /**
     * 修改记录
     */
    @PutMapping
    public ApiResult<Void> edit(@RequestBody @Validated DeviceType deviceType)
    {
        deviceTypeService.update(deviceType);
        return ApiResult.success();
    }

    /**
     * 记录查询
     * @param current 页数
     * @param size 条数
     * @param name 名称
     * @param typeValue 备注
     * @return
     */
    @GetMapping("/list")
    public ApiResult<CommonPage<DeviceType>> list(@RequestParam Integer current,
                                                  @RequestParam Integer size,
                                                  @RequestParam(required = false) String name,
                                                  @RequestParam(required = false) String typeValue)
    {
        return ApiResult.success(deviceTypeService.list(current,size,name,typeValue));
    }

    /**
     * 导出记录
     * @param name 名称
     * @param typeValue 备注
     * @return
     */
    @GetMapping("/export")
    public ApiResult<String> export(@RequestParam(required = false) String name, @RequestParam(required = false) String typeValue){
        return ApiResult.success(deviceTypeService.export(name,typeValue));
    }

    /**
     * 查询全部
     * @param name 名称
     * @param typeValue typeValue
     * @return
     */
    @GetMapping("/all")
    public ApiResult<List<DeviceType>> all(@RequestParam(required = false) String name,
                                           @RequestParam(required = false) String typeValue){
        return ApiResult.success(deviceTypeService.all(name,typeValue));
    }


    /**
     * 删除记录
     */
    @DeleteMapping("/{id}")
    public ApiResult<Void> remove(@PathVariable("id") Integer id)
    {
        deviceTypeService.remove(id);
        return ApiResult.success();
    }
}

