package com.usky.dm.controller.web;


import com.usky.common.core.bean.ApiResult;
import com.usky.common.core.bean.CommonPage;
import com.usky.dm.domain.EventManage;
import com.usky.dm.service.EventManageService;
import com.usky.dm.service.vo.AlertCheckVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;

/**
 * 事件管理
 *
 * @author ya
 * @since 2022-02-22
 */
@RestController
@RequestMapping("/eventManage")
public class EventManageController {

    @Autowired
    private EventManageService eventManageService;


    /**
     * 新增
     * @param eventManage
     * @return
     */
    @PostMapping
    public ApiResult<Void> add(@RequestBody EventManage eventManage){
        eventManageService.add(eventManage);
        return ApiResult.success();
    }

    /**
     * 修改
     */
    @PutMapping
    public ApiResult<Void> edit(@RequestBody EventManage eventManage)
    {
        eventManageService.update(eventManage);
        return ApiResult.success();
    }

    /**
     * 记录查询
     * @param current 页数
     * @param size 条数
     * @param name 名称
     * @param type 地址
     * @return
     */
    @GetMapping("/list")
    public ApiResult<CommonPage<EventManage>> list(@RequestParam Integer current,
                                                   @RequestParam Integer size,
                                                   @RequestParam(required = false) String name,
                                                   @RequestParam(required = false) String type,
                                                   @RequestParam(required = false) Date startTime,
                                                   @RequestParam(required = false) Date endTime)
    {
        return ApiResult.success(eventManageService.list(current,size,name,type,startTime,endTime));
    }


    /**
     * 导出记录
     * @param name 名称
     * @param type 地址
     * @return
     */
    @GetMapping("/export")
    public ApiResult<String> export(@RequestParam(required = false) String name,
                                                   @RequestParam(required = false) String type,
                                    @RequestParam(required = false) Date startTime,
                                    @RequestParam(required = false) Date endTime)
    {
        return ApiResult.success(eventManageService.export(name,type,startTime,endTime));
    }



    /**
     * 事件汇总
     * @return
     */
    @GetMapping("/collect")
    public ApiResult<AlertCheckVO> getCollect(@RequestParam(required = false) Integer type){
        return ApiResult.success(eventManageService.getCollect(type));
    }

    /**
     * 删除
     */
    @DeleteMapping("/{id}")
    public ApiResult<Void> remove(@PathVariable("id") Integer id)
    {
        eventManageService.remove(id);
        return ApiResult.success();
    }
}

