package com.usky.dm.controller.web;


import com.usky.common.core.bean.ApiResult;
import com.usky.common.core.bean.CommonPage;
import com.usky.dm.domain.Facility;
import com.usky.dm.service.FacilityService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Date;

/**
 * 资产表
 *
 * @author ya
 * @since 2022-02-22
 */
@RestController
@RequestMapping("/facility")
public class FacilityController {

    @Autowired
    private FacilityService facilityService;


    /**
     * 新增记录
     * @param facility
     * @return
     */
    @PostMapping
    public ApiResult<Void> add(@RequestBody @Validated Facility facility){
        facility.setCreatTime(new Date());
        facility.setStatus(0);
        facilityService.add(facility);
        return ApiResult.success();
    }

    /**
     * 修改记录
     */
    @PutMapping
    public ApiResult<Void> edit(@RequestBody @Validated Facility facility)
    {
        facilityService.update(facility);
        return ApiResult.success();
    }

    /**
     * 记录查询
     * @param current 页数
     * @param size 条数
     * @param name 名称
     * @param address 地址
     * @return
     */
    @GetMapping("/list")
    public ApiResult<CommonPage<Facility>> list(@RequestParam Integer current,
                                                @RequestParam Integer size,
                                                @RequestParam(required = false) String name,
                                                @RequestParam(required = false) String address)
    {
        return ApiResult.success(facilityService.list(current,size,name,address));
    }

    /**
     * 导出记录
     * @param name 名称
     * @param address 地址
     * @return
     */
    @GetMapping("/export")
    public ApiResult<String> export(String name,String address){
        return ApiResult.success(facilityService.export(name,address));
    }

    /**
     * 删除记录
     */
    @DeleteMapping("/{id}")
    public ApiResult<Void> remove(@PathVariable("id") Integer id)
    {
        facilityService.remove(id);
        return ApiResult.success();
    }
}

