package com.usky.dm.controller.web;


import com.usky.common.core.bean.ApiResult;
import com.usky.common.core.bean.CommonPage;
import com.usky.dm.domain.Floor;
import com.usky.dm.service.FloorService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * 楼层
 * @author ya
 * @since 2022-02-22
 */
@RestController
@RequestMapping("/floor")
public class FloorController {

    @Autowired
    private FloorService floorService;

    /**
     * 建筑新增
     * @param floor
     * @return
     */
    @PostMapping
    public ApiResult<Boolean> add(@RequestBody Floor floor){
        return ApiResult.success(floorService.add(floor));
    }

    /**
     * 建筑修改
     * @return
     */
    @PutMapping
    public ApiResult<Boolean> update(@RequestBody Floor floor){
        return ApiResult.success(floorService.update(floor));
    }

    /**
     * 删除记录
     * @param id
     * @return
     */
    @DeleteMapping("/{id}")
    public ApiResult<Boolean> delete(@PathVariable("id") Long id){
        return ApiResult.success(floorService.delete(id));
    }

    /**
     * 楼层查询
     * @param buildName 建筑名称
     * @param name 楼层名称
     * @param buildId 建筑id
     * @param current 页数
     * @param size 条数
     * @return
     */
    @GetMapping("list")
    public ApiResult<CommonPage<Floor>> page(@RequestParam(required = false) String buildName,
                                             @RequestParam(required = false) String name,
                                             @RequestParam(required = false) Long buildId,
                                             @RequestParam Integer current,
                                             @RequestParam Integer size){
        return ApiResult.success(floorService.page(name,buildName,buildId,current,size));
    }
}

