package com.usky.dm.controller.web;


import com.usky.common.core.bean.ApiResult;
import com.usky.common.core.bean.CommonPage;
import com.usky.dm.domain.InspectionPoint;
import com.usky.dm.service.InspectionPointService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * 维保点位
 *
 * @author ya
 * @since 2022-02-22
 */
@RestController
@RequestMapping("/inspectionPoint")
public class InspectionPointController {

    @Autowired
    private InspectionPointService inspectionPointService;


    /**
     * 新增记录
     * @param inspectionPoint
     * @return
     */
    @PostMapping
    public ApiResult<Void> add(@RequestBody InspectionPoint inspectionPoint){
        inspectionPointService.add(inspectionPoint);
        return ApiResult.success();
    }

    /**
     * 修改记录
     */
    @PutMapping
    public ApiResult<Void> edit(@RequestBody InspectionPoint inspectionPoint)
    {
        inspectionPointService.update(inspectionPoint);
        return ApiResult.success();
    }

    /**
     * 记录查询
     * @param current 页数
     * @param size 条数
     * @param name 名称
     * @param address 地址
     * @return
     */
    @GetMapping("/list")
    public ApiResult<CommonPage<InspectionPoint>> list(@RequestParam Integer current,
                                                       @RequestParam Integer size,
                                                       @RequestParam(required = false) String name,
                                                       @RequestParam(required = false) String address)
    {
        return ApiResult.success(inspectionPointService.list(current,size,name,address));
    }


    /**
     * 导出记录
     * @param name 名称
     * @param address 地址
     * @return
     */
    @GetMapping("/export")
    public ApiResult<String> export(@RequestParam(required = false) String name,
                                    @RequestParam(required = false) String address){
        return ApiResult.success(inspectionPointService.export(name,address));
    }


    /**
     * 删除记录
     */
    @DeleteMapping("/{id}")
    public ApiResult<Void> remove(@PathVariable("id") Integer id)
    {
        inspectionPointService.remove(id);
        return ApiResult.success();
    }
}

