package com.usky.dm.controller.web;


import com.usky.common.core.bean.ApiResult;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.dm.domain.Product;
import com.usky.dm.service.ProductService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * 产品
 * @author ya
 * @since 2022-03-31
 */
@RestController
@RequestMapping("/product")
public class ProductController {

    @Autowired
    private ProductService productService;

    /**
     * 新增记录
     * @param product
     * @return
     */
    @PostMapping
    public ApiResult add(@RequestBody Product product){
        if (productService.checkNameUnique(product))
        {
            throw new BusinessException("新增产品信息'" + product.getName() + "'失败，产品信息已存在");
        }
        return ApiResult.success(productService.add(product));
    }

    /**
     * 修改记录
     */
    @PutMapping
    public ApiResult edit(@RequestBody Product product)
    {
        return ApiResult.success(productService.update(product));
    }

    /**
     * 记录查询
     * @param current 页数
     * @param size 条数
     * @param name 名称
     * @return
     */
    @GetMapping("/list")
    public ApiResult<CommonPage<Product>> list(@RequestParam Integer current,
                                               @RequestParam Integer size,
                                               @RequestParam(required = false) String name)
    {
        return ApiResult.success(productService.list(current, size, name));
    }


    /**
     * 删除记录
     */
    @DeleteMapping("/{id}")
    public ApiResult remove(@PathVariable("id") Long id)
    {
        return ApiResult.success(productService.remove(id));
    }
}

