package com.usky.dm.controller.web;


import com.usky.common.core.bean.ApiResult;
import com.usky.common.core.bean.CommonPage;
import com.usky.dm.domain.Room;
import com.usky.dm.service.RoomService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * 房间
 * @author ya
 * @since 2022-02-22
 */
@RestController
@RequestMapping("/room")
public class RoomController {


    @Autowired
    private RoomService roomService;

    /**
     * 房间新增
     * @param room
     * @return
     */
    @PostMapping
    public ApiResult<Boolean> add(@RequestBody Room room){
        return ApiResult.success(roomService.add(room));
    }

    /**
     * 房间修改
     * @return
     */
    @PutMapping
    public ApiResult<Boolean> update(@RequestBody Room room){
        return ApiResult.success(roomService.update(room));
    }

    /**
     * 房间删除
     * @param id
     * @return
     */
    @DeleteMapping("/{id}")
    public ApiResult<Boolean> delete(@PathVariable("id") Long id){
        return ApiResult.success(roomService.delete(id));
    }

    /**
     * 房间查询
     * @param name 名称
     * @param floorName 楼层名称
     * @param floorId 楼层id
     * @param current 页数
     * @param size 条数
     * @param buildName 建筑名称
     * @return
     */
    @GetMapping("list")
    public ApiResult<CommonPage<Room>> page(@RequestParam(required = false) String floorName,
                                            @RequestParam(required = false) String name,
                                            @RequestParam(required = false) Long floorId,
                                            @RequestParam Integer current,
                                            @RequestParam Integer size,
                                            @RequestParam(required = false) String buildName){
        return ApiResult.success(roomService.page(name,floorName,floorId,current,size,buildName));
    }
}

