package com.usky.dm.controller.web;


import com.usky.common.core.bean.ApiResult;
import com.usky.common.core.bean.CommonPage;
import com.usky.dm.domain.WorkOrder;
import com.usky.dm.mapper.vo.WorkOrderRequest;
import com.usky.dm.service.WorkOrderService;
import com.usky.dm.service.vo.AlertCheckVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * 工单
 *
 * @author ya
 * @since 2022-02-22
 */
@RestController
@RequestMapping("/workOrder")
public class WorkOrderController {


    @Autowired
    private WorkOrderService workOrderService;


    /**
     * 新增记录
     * @param workOrder
     * @return
     */
    @PostMapping
    public ApiResult<Void> add(@RequestBody WorkOrder workOrder){
        workOrderService.add(workOrder);
        return ApiResult.success();
    }

    /**
     * 修改记录
     */
    @PutMapping
    public ApiResult<Void> edit(@Validated @RequestBody WorkOrder workOrder)
    {
        workOrderService.update(workOrder);
        return ApiResult.success();
    }

    /**
     * 记录查询
     * @param workOrderRequest 工单
     * @return
     */
    @PostMapping("/list")
    public ApiResult<CommonPage<WorkOrder>> list(@RequestBody WorkOrderRequest workOrderRequest)
    {
        return ApiResult.success(workOrderService.list(workOrderRequest));
    }


    /**
     * 导出记录
     * @param workOrderRequest 工单
     * @return
     */
    @PostMapping("/export")
    public ApiResult<String> export(@RequestBody WorkOrderRequest workOrderRequest)
    {
        return ApiResult.success(workOrderService.export(workOrderRequest));
    }

    /**
     * 工单汇总
     * @return
     */
    @GetMapping("/collect")
    public ApiResult<AlertCheckVO> getCollect(){
        return ApiResult.success(workOrderService.getCollect());
    }


    /**
     * 删除记录
     */
    @DeleteMapping("/{id}")
    public ApiResult<Void> remove(@PathVariable("id") Integer id)
    {
        return ApiResult.success();
    }


    /**
     * 查看工单详情
     * @param type 类型
     * @param businessId 业务编号
     * @return
     */
    @GetMapping("/oneByType")
    public ApiResult<WorkOrder> oneByType(@RequestParam Integer type, @RequestParam String businessId){
        return ApiResult.success(workOrderService.oneByTypeAndBusiness(type,businessId));
    }
}

