package com.usky.dm.domain;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

/**
 * <p>
 * 资产表
 * </p>
 *
 * @author ya
 * @since 2022-02-22
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
public class Facility implements Serializable {

    private static final long serialVersionUID=1L;

    /**
     * 主键id
     */
    @TableId(value = "id", type = IdType.AUTO)
    private Integer id;

    /**
     * 资产编号
     */
    @NotBlank(message = "资产编号不能为空")
    @Size(min = 0, max = 100, message = "资产编号不能超过100个字符")
    private String facilityNo;

    /**
     * 资产名称
     */
    @NotBlank(message = "资产名称不能为空")
    @Size(min = 0, max = 100, message = "设备名称不能超过100个字符")
    private String facilityName;

    /**
     * 资产类别
     */
    @NotNull(message = "资产类别不能为空")
    private Integer facilityType;

    /**
     * 使用单位
     */
    private String company;

    /**
     * 使用不部门
     */
    private String department;

    /**
     * 设备型号
     */
    private String facilityModel;

    /**
     * 计量单位
     */
    private String unit;

    /**
     * 负责人
     */
    private String man;

    /**
     * 购入方式
     */
    private String purchase;

    /**
     * 数量
     */
    @Min(value = 1,message = "资产数量不能小于1")
    private Long number;

    /**
     * 单价
     */
    private BigDecimal price;

    /**
     * 存放地点
     */
    private String addrs;

    /**
     * 供应商
     */
    private String supplier;

    /**
     * 入账日期
     */
    private Date creatTime;

    /**
     * 购买日期
     */
    private Date buyTime;

    /**
     * 采购编号
     */
    @NotBlank(message = "采购编号不能为空")
    @Size(min = 0, max = 200, message = "采购编号不能超过200个字符")
    private String buyNo;

    /**
     * 发票编号
     */
    @NotBlank(message = "发票编号不能为空")
    @Size(min = 0, max = 255, message = "发票编号不能超过255个字符")
    private String invoice;

    /**
     * 登记编号 uuid
     */
    private String register;

    /**
     * 资产用途
     */
    private String purpose;

    /**
     * 保修天数
     */
    private Integer warranty;

    /**
     * 保修周期
     */
    private Integer warrantyPeriod;

    /**
     * 已使用年限
     */
    private String usedData;

    /**
     * 删除状态 0 删除 1未删除
     */
    private Integer status;

    /**
     * 二维码地址
     */
    private String qrurl;

    @TableField(exist = false)
    private String companyName;

}
