package com.usky.dm.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.core.util.JsonUtils;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.dm.domain.BaCwInfo;
import com.usky.dm.domain.WorkOrder;
import com.usky.dm.mapper.BaCwInfoMapper;
import com.usky.dm.service.BaCwInfoService;
import com.usky.dm.service.WorkOrderService;
import com.usky.dm.service.enums.WorkOrderType;
import com.usky.dm.service.vo.BaCwCollectVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author ya
 * @since 2022-05-25
 */
@Service
public class BaCwInfoServiceImpl extends AbstractCrudService<BaCwInfoMapper, BaCwInfo> implements BaCwInfoService {

    @Autowired
    private WorkOrderService workOrderService;


//    @Autowired
//    private SysConfigMapper sysConfigMapper;
//
//    @Override
//    public void addByConfig() {
//        List<BaCwInfo> addList = new ArrayList<>();
//        LambdaQueryWrapper<BaCwInfo> queryWrapper = Wrappers.lambdaQuery();
//        queryWrapper.select(BaCwInfo::getDeviceName);
//        List<BaCwInfo> baCwInfoList = this.list(queryWrapper);
//        SysConfig config = new SysConfig();
//        config.setConfigKey("ba.wc.mapping");
//        SysConfig sysConfig = sysConfigMapper.selectConfig(config);
//        BaCwVO baCwVO = JsonUtils.fromJson(sysConfig.getConfigValue(), BaCwVO.class);
//        for (BaCwFloorVO baCwFloorVO:baCwVO.getWClist()) {
//            for (BaCwDetailVO baCwDetailVO:baCwFloorVO.getCwlist()) {
//                boolean present = baCwInfoList.stream()
//                        .anyMatch(baCwInfo -> baCwInfo.getDeviceName().equals(baCwDetailVO.getCode()));
//                if (!present && StringUtils.isNotBlank(baCwDetailVO.getCode())){
//                    BaCwInfo baCwInfo = new BaCwInfo();
//                    baCwInfo.setDeviceName(baCwDetailVO.getCode());
//                    baCwInfo.setWcType(baCwFloorVO.getType());
//                    baCwInfo.setFloor(baCwFloorVO.getFloor().toString());
//                    baCwInfo.setWcCode(baCwDetailVO.getId());
//                    baCwInfo.setStatus("0");
//                    addList.add(baCwInfo);
//                }
//            }
//        }
//        this.saveBatch(addList);
//    }

    @Override
    public CommonPage<BaCwInfo> list(Integer current, Integer size, Date startTime, Date endTime,String status,String floor,
                                     String type,Boolean isAlert,String name) {
        IPage<BaCwInfo> page = new Page<>(current,size);
        LambdaQueryWrapper<BaCwInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper
                .between(null != startTime && null != endTime,BaCwInfo::getCreateTime,startTime,endTime)
                .eq(StringUtils.isNotBlank(status),BaCwInfo::getStatus,status)
                .eq(StringUtils.isNotBlank(floor),BaCwInfo::getFloor,floor)
                .eq(StringUtils.isNotBlank(type),BaCwInfo::getWcType,type)
                .eq(null != isAlert,BaCwInfo::getIsAlert,isAlert)
                .eq(StringUtils.isNotBlank(name),BaCwInfo::getDeviceName,name)
                .orderByDesc(BaCwInfo::getCreateTime);
        page = this.page(page, queryWrapper);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getSize(),page.getCurrent());
    }


    @Transactional(rollbackFor = Exception.class)
    @Override
    public void addAlert(WorkOrder wo) {
        BaCwInfo bc = new BaCwInfo();
        bc.setDeviceName(wo.getBusinessId());
        bc = getByName(bc);
        if (bc.getIsAlert()){
            throw new BusinessException("该厕位已提交厕位工单");
        }
        bc.setCreateTime(new Date());
        bc.setStatus("0");
        bc.setIsAlert(true);
        this.updateById(bc);
        WorkOrder workOrder1 = workOrderService.generateWorkerOrder(workOrder -> {
            workOrder.setAddPeople(wo.getAddPeople());
            workOrder.setWorkOrderName(WorkOrderType.BA_TOILET_ALERT.getName());
            workOrder.setWorkOrderContent(wo.getWorkOrderContent());
            workOrder.setWorkOrderType(WorkOrderType.BA_TOILET_ALERT.getCode());
            workOrder.setBusinessId(wo.getBusinessId());
            return workOrder;
        });
        workOrderService.save(workOrder1);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void checkAlert(WorkOrder workOrder) {
        workOrder.setWorkOrderType(WorkOrderType.BA_TOILET_ALERT.getCode());
        workOrderService.checkWorkOrder(workOrder,wo -> {
            if (StringUtils.isNotBlank(wo.getBusinessId())){
                BaCwInfo baCwInfo = new BaCwInfo();
                baCwInfo.setDeviceName(wo.getBusinessId());
                BaCwInfo bc = getByName(baCwInfo);
                bc.setIsAlert(false);
                return this.updateById(bc);
            }else {
                return null;
            }
        });
    }

    @Override
    public BaCwCollectVO collect(String floor, String type) {
        BaCwCollectVO baCwCollectVO = new BaCwCollectVO();
        QueryWrapper<BaCwInfo> query = Wrappers.query();
        query.select("count(*) deviceCount","status as clzt")
                .eq(StringUtils.isNotBlank(floor),"floor",floor)
                .eq(StringUtils.isNotBlank(type),"wc_type",type)
                .groupBy("clzt");
        List<Map<String, Object>> mapList = this.listMaps(query);
        baCwCollectVO.setLeisure(getCount(mapList,"0"));
        baCwCollectVO.setOccupy(getCount(mapList,"1"));


        QueryWrapper<BaCwInfo> queryByAlert = Wrappers.query();
        queryByAlert.select("count(*) deviceCount","is_alert as clzt")
                .eq(StringUtils.isNotBlank(floor),"floor",floor)
                .eq(StringUtils.isNotBlank(type),"wc_type",type)
                .groupBy("clzt");
        baCwCollectVO.setAlert(getCount(this.listMaps(queryByAlert),"1"));
        return baCwCollectVO;
    }

    @Override
    public List<BaCwCollectVO> collectByFloor(String floor, String type) {
        QueryWrapper<BaCwInfo> query = Wrappers.query();
        query.select("floor as floor","wc_type as wcType",
                "sum(if(status = '0', 1, 0)) as leisure",
                "sum(if(status = '1', 1, 0)) as occupy",
                "sum(if(is_alert = '1', 1, 0)) as alert")
                .eq(StringUtils.isNotBlank(floor),"floor",floor)
                .eq(StringUtils.isNotBlank(type),"wc_type",type)
                .groupBy("floor,wc_type");
        List<Map<String, Object>> mapList = this.listMaps(query);
        String jsonStr = JsonUtils.toJson(mapList);
        return JSON.parseArray(jsonStr, BaCwCollectVO.class);
    }

    public Integer getCount(List<Map<String, Object>> mapList,String status){
        return mapList.stream()
                .filter(mp -> mp.get("clzt").toString().equals(status))
                .map(mp -> Integer.parseInt(mp.get("deviceCount").toString()))
                .findAny().orElse(0);
    }

    public BaCwInfo getByName(BaCwInfo baCwInfo){
        LambdaQueryWrapper<BaCwInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(BaCwInfo::getDeviceName,baCwInfo.getDeviceName());
        return this.getOne(queryWrapper);
    }
}
