package com.usky.dm.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.core.util.StringUtils;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.dm.domain.Building;
import com.usky.dm.domain.Floor;
import com.usky.dm.mapper.BuildingMapper;
import com.usky.dm.service.BuildingService;
import com.usky.dm.service.FloorService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Objects;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author ya
 * @since 2022-02-22
 */
@Service
public class BuildingServiceImpl extends AbstractCrudService<BuildingMapper, Building> implements BuildingService {


    @Autowired
    private FloorService floorService;

    @Override
    public Boolean add(Building building)
    {
        building.setAddMan(SecurityUtils.getUsername());
        if (checkNameUnique(building)){
            throw new BusinessException("建筑名称不能重复");
        }
        this.save(building);
        return true;
    }

    @Override
    public Boolean update(Building building) {

        if (checkNameUnique(building)){
            throw new BusinessException("建筑名称不能重复");
        }
        this.updateById(building);
        return true;
    }


    @Override
    public Boolean checkNameUnique(Building building) {
        Long id =  null == building.getId() ? -1 : building.getId();
        LambdaQueryWrapper<Building> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(Building::getBuildName,building.getBuildName())
                .eq(Building::getData1,"0");
        Building one = this.getOne(queryWrapper);
        if (Objects.nonNull(one) && one.getId() != id.longValue())
        {
            return true;
        }
        return false;
    }

    @Override
    public Boolean delete(Long id) {
        LambdaQueryWrapper<Floor> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(Floor::getStatus,0)
                .eq(Floor::getBuildId,id);
        int count = floorService.count(queryWrapper);
        if (count > 0){
            throw new BusinessException("请先删除下属楼层信息");
        }
        Building tbBuilding = new Building();
        tbBuilding.setId(id);
        tbBuilding.setData1("1");
        return this.updateById(tbBuilding);
    }

    @Override
    public CommonPage<Building> page(Integer current, Integer size,String name) {
        IPage<Building> page = new Page<>(current, size);
        LambdaQueryWrapper<Building> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper
                .like(StringUtils.isNotBlank(name),Building::getBuildName,name)
                .eq(Building::getData1,"0")
                .orderByDesc(Building::getId);
        page =  this.page(page,queryWrapper);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getSize(),page.getCurrent());
    }
}
