package com.usky.dm.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.core.util.BeanMapperUtils;
import com.usky.common.core.util.StringUtils;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.dm.domain.CommunicationApi;
import com.usky.dm.mapper.CommunicationApiMapper;
import com.usky.dm.service.CommunicationApiService;
import com.usky.dm.service.util.ExportExcelUtil;
import com.usky.dm.service.vo.CApiExport;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Objects;

/**
 * <p>
 * 通信接口 服务实现类
 * </p>
 *
 * @author ya
 * @since 2022-02-22
 */
@Service
public class CommunicationApiServiceImpl extends AbstractCrudService<CommunicationApiMapper, CommunicationApi> implements CommunicationApiService {

    @Override
    public boolean add(CommunicationApi communicationApi) {
        communicationApi.setAddPeople(SecurityUtils.getUsername());
        if (checkNameUnique(communicationApi)){
            throw new BusinessException("通信接口名称不能重复");
        }
        this.save(communicationApi);
        return true;
    }

    @Override
    public boolean update(CommunicationApi communicationApi) {
        if (checkNameUnique(communicationApi)){
            throw new BusinessException("通信接口名称不能重复");
        }
        this.updateById(communicationApi);
        return true;
    }

    @Override
    public Boolean checkNameUnique(CommunicationApi communicationApi) {
        Integer id =  null == communicationApi.getId() ? -1 : communicationApi.getId();
        LambdaQueryWrapper<CommunicationApi> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(CommunicationApi::getInterfaceName,communicationApi.getInterfaceName())
                .eq(CommunicationApi::getEState,0);
        CommunicationApi one = this.getOne(queryWrapper);
        if (Objects.nonNull(one) && one.getId() != id.longValue())
        {
            return true;
        }
        return false;
    }

    @Override
    public CommonPage<CommunicationApi> list(Integer current, Integer size, String storeName, String transferType) {
        IPage<CommunicationApi> page = new Page<>(current,size);
        LambdaQueryWrapper<CommunicationApi> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper
                .like(StringUtils.isNotBlank(storeName),CommunicationApi::getBusiness,storeName)
                .like(StringUtils.isNotBlank(transferType),CommunicationApi::getInterfaceTransmissionMode,transferType)
                .eq(CommunicationApi::getEState,0)
                .orderByDesc(CommunicationApi::getId);
        page = this.page(page, queryWrapper);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getSize(),page.getCurrent());
    }

    @Override
    public String export(String storeName, String transferType) {
        ExportParams params = new ExportParams(null, "通信接口");
        return ExportExcelUtil.export("通信接口", param -> ExcelExportUtil.exportBigExcel(params, CApiExport.class,
                (o, i) -> {
                    CommonPage<CommunicationApi> page = this.list(i,30,storeName,transferType);
                    return new ArrayList<>(BeanMapperUtils.mapList(page.getRecords(), CommunicationApi.class, CApiExport.class));
                }, null));
    }

    @Override
    public boolean remove(Integer id) {
        CommunicationApi communicationApi= new CommunicationApi();
        communicationApi.setId(id);
        communicationApi.setEState(1);
        return this.updateById(communicationApi);
    }
}
