package com.usky.dm.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.core.util.StringUtils;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.dm.domain.Company;
import com.usky.dm.mapper.CompanyMapper;
import com.usky.dm.service.CompanyService;
import org.springframework.stereotype.Service;

import java.util.Objects;

/**
 * <p>
 * 单位表 服务实现类
 * </p>
 *
 * @author ya
 * @since 2022-02-22
 */
@Service
public class CompanyServiceImpl extends AbstractCrudService<CompanyMapper, Company> implements CompanyService {

    @Override
    public CommonPage<Company> list(Integer current, Integer size, String name) {
        IPage<Company> page = new Page<>(current,size);
        LambdaQueryWrapper<Company> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper
                .like(StringUtils.isNotBlank(name),Company::getOwnerName,name)
                .orderByDesc(Company::getId);
        page = this.page(page, queryWrapper);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getSize(),page.getCurrent());
    }

    @Override
    public boolean add(Company company) {
        if (checkNameUnique(company)){
            throw new BusinessException("单位名称不能重复");
        }
        this.save(company);
        return true;
    }

    @Override
    public boolean update(Company company) {
        if (checkNameUnique(company)){
            throw new BusinessException("单位名称不能重复");
        }
        this.updateById(company);
        return true;
    }

    @Override
    public Boolean checkNameUnique(Company company) {
        Integer id =  null == company.getId() ? -1 : company.getId();
        LambdaQueryWrapper<Company> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(Company::getOwnerName,company.getOwnerName())
                .eq(Company::getData1,"0");
        Company one = this.getOne(queryWrapper);
        if (Objects.nonNull(one) && one.getId() != id.longValue())
        {
            return true;
        }
        return false;
    }

    @Override
    public boolean remove(Integer id) {
        Company company= new Company();
        company.setId(id);
        company.setData1("1");
        return this.updateById(company);
    }

    @Override
    public Company getDefault() {
        LambdaQueryWrapper<Company> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(Company::getOwnerId,"10316");
        return this.getOne(queryWrapper);
    }
}
