package com.usky.dm.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.core.util.BeanMapperUtils;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.dm.domain.DeviceChannel;
import com.usky.dm.mapper.DeviceChannelMapper;
import com.usky.dm.service.DeviceChannelService;
import com.usky.dm.service.util.ExportExcelUtil;
import com.usky.dm.service.vo.DeviceChannelExport;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.Objects;

/**
 * <p>
 *  监测通道
 * </p>
 *
 * @author ya
 * @since 2022-02-22
 */
@Service
public class DeviceChannelServiceImpl extends AbstractCrudService<DeviceChannelMapper, DeviceChannel> implements DeviceChannelService {

    @Override
    public boolean add(DeviceChannel deviceChannel) {
        deviceChannel.setAddTime(new Date());
        deviceChannel.setAddPeople(SecurityUtils.getUsername());
        if (checkNameUnique(deviceChannel)){
            throw new BusinessException("监测通道名称不能重复");
        }
        this.save(deviceChannel);
        return true;
    }

    @Override
    public boolean update(DeviceChannel deviceChannel) {
        if (checkNameUnique(deviceChannel)){
            throw new BusinessException("监测通道名称不能重复");
        }
        this.updateById(deviceChannel);
        return true;
    }

    @Override
    public Boolean checkNameUnique(DeviceChannel deviceChannel) {
        Integer id =  null == deviceChannel.getId() ? -1 : deviceChannel.getId();
        LambdaQueryWrapper<DeviceChannel> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(DeviceChannel::getPassagewayName,deviceChannel.getPassagewayName())
                .eq(DeviceChannel::getEState,0);
        DeviceChannel one = this.getOne(queryWrapper);
        if (Objects.nonNull(one) && one.getId() != id.longValue())
        {
            return true;
        }
        return false;
    }

    @Override
    public CommonPage<DeviceChannel> list(Integer current, Integer size, String name, Integer dwType) {
        IPage<DeviceChannel> page = new Page<>(current,size);
        page = baseMapper.page(page,name,dwType);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getSize(),page.getCurrent());
    }

    @Override
    public String export(String name, Integer dwType) {
        ExportParams params = new ExportParams(null, "监测通道");
        return ExportExcelUtil.export("监测通道", param -> ExcelExportUtil.exportBigExcel(params, DeviceChannelExport.class,
                (o, i) -> {
                    CommonPage<DeviceChannel> page = this.list(i,30,name,dwType);
                    return new ArrayList<>(BeanMapperUtils.mapList(page.getRecords(), DeviceChannel.class, DeviceChannelExport.class));
                }, null));
    }

    @Override
    public boolean remove(Integer id) {
        DeviceChannel deviceChannel= new DeviceChannel();
        deviceChannel.setId(id);
        deviceChannel.setEState(1);
        return this.updateById(deviceChannel);
    }
}
