package com.usky.dm.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.core.util.BeanMapperUtils;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.dm.domain.Device;
import com.usky.dm.domain.DeviceStatus;
import com.usky.dm.mapper.DeviceMapper;
import com.usky.dm.mapper.vo.DeviceCollectVO;
import com.usky.dm.mapper.vo.DeviceRequestVO;
import com.usky.dm.service.DeviceService;
import com.usky.dm.service.DeviceStatusService;
import com.usky.dm.service.enums.DeviceCollectCode;
import com.usky.dm.service.util.AlertCheckUtil;
import com.usky.dm.service.util.ExportExcelUtil;
import com.usky.dm.service.vo.DeviceExport;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;


/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author ya
 * @since 2022-02-22
 */
@Service
public class DeviceServiceImpl extends AbstractCrudService<DeviceMapper, Device> implements DeviceService {

    @Autowired
    private DeviceStatusService deviceStatusService;

    @Override
    public boolean add(Device device) {
        device.setInstallTime(new Date());
        device.setInstallMan(SecurityUtils.getUsername());
        device.setHls("0");
        if (checkNameUnique(device)){
            throw new BusinessException("设备编号不能重复");
        }
        this.save(device);
        deviceStatusService.save(addDeviceStatus(device));
        return true;
    }

    public DeviceStatus addDeviceStatus(Device device){
        DeviceStatus deviceStatus = new DeviceStatus();
        deviceStatus.setDeviceid(device.getOwnerCode());
        deviceStatus.setDeviceno(0);
        deviceStatus.setDevicestatus(0);
        deviceStatus.setStatustime(new Date());
        deviceStatus.setFEvt("0");
        deviceStatus.setCEvt("0");
        deviceStatus.setWEvt("0");
        deviceStatus.setTrueStatus("0");
        return deviceStatus;
    }

    @Override
    public boolean update(Device device) {
        if (checkNameUnique(device)){
            throw new BusinessException("设备编号不能重复");
        }
        this.updateById(device);
        return true;
    }

    @Override
    public Boolean checkNameUnique(Device device) {
        Integer id =  null == device.getId() ? -1 : device.getId();
        LambdaQueryWrapper<Device> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(Device::getOwnerCode,device.getOwnerCode())
                .eq(Device::getHls,"0");
        Device one = this.getOne(queryWrapper);
        if (Objects.nonNull(one) && one.getId() != id.longValue())
        {
            return true;
        }
        return false;
    }

    @Override
    public CommonPage<Device> list(DeviceRequestVO deviceRequestVO) {
        IPage<Device> page = new Page<>(deviceRequestVO.getCurrent(),deviceRequestVO.getSize());
        List<Integer> type = AlertCheckUtil.getType(deviceRequestVO.getBigDeviceType());
        deviceRequestVO.setDeviceTypes(type);
        page = baseMapper.list(page,deviceRequestVO);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getSize(),page.getCurrent());
    }

    @Override
    public String export(DeviceRequestVO deviceRequestVO) {
        ExportParams params = new ExportParams(null, "设备");
        return ExportExcelUtil.export("设备", param -> ExcelExportUtil.exportBigExcel(params, DeviceExport.class,
                (o, i) -> {
                    deviceRequestVO.setCurrent(i);
                    deviceRequestVO.setSize(30);
                    CommonPage<Device> page = this.list(deviceRequestVO);
                    return new ArrayList<>(BeanMapperUtils.mapList(page.getRecords(), Device.class, DeviceExport.class));
                }, null));
    }

    @Override
    public boolean remove(Integer id) {
        Device device= new Device();
        device.setId(id);
        device.setHls("1");
        return this.updateById(device);
    }

    @Override
    public List<DeviceCollectVO> getCollect(Integer isLine) {
        List<DeviceCollectVO> reList = new ArrayList<>();
        List<DeviceCollectVO> list = baseMapper.getCollect();
        List<DeviceCollectVO> collect = list.stream().filter(dv -> null == isLine || (1 == isLine && 0 != dv.getDeviceStatus())).collect(Collectors.toList());
        for (DeviceCollectCode deviceCollectCode: DeviceCollectCode.values()) {
            DeviceCollectVO deviceCollectVO = new DeviceCollectVO();
            deviceCollectVO.setDeviceTypeS(deviceCollectCode.getName());
            deviceCollectVO.setDeviceCount(AlertCheckUtil.getCountByType(collect,deviceCollectCode));
            reList.add(deviceCollectVO);
        }
        return reList;
    }

}
