package com.usky.dm.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.core.util.BeanMapperUtils;
import com.usky.common.core.util.StringUtils;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.dm.domain.Device;
import com.usky.dm.domain.DeviceType;
import com.usky.dm.mapper.DeviceTypeMapper;
import com.usky.dm.service.DeviceService;
import com.usky.dm.service.DeviceTypeService;
import com.usky.dm.service.util.ExportExcelUtil;
import com.usky.dm.service.vo.DeviceTypeExport;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author ya
 * @since 2022-02-22
 */
@Service
public class DeviceTypeServiceImpl extends AbstractCrudService<DeviceTypeMapper, DeviceType> implements DeviceTypeService {

    @Autowired
    private DeviceService deviceService;

    @Override
    public boolean add(DeviceType deviceType) {
        deviceType.setAddPeople(SecurityUtils.getUsername());
        if (checkNameUnique(deviceType)){
            throw new BusinessException("设备类型不能重复");
        }
        if (checkTypeUnique(deviceType)){
            throw new BusinessException("设备类型值不能重复");
        }
        this.save(deviceType);
        return true;
    }

    @Override
    public boolean update(DeviceType deviceType) {
        if (checkNameUnique(deviceType)){
            throw new BusinessException("设备类型不能重复");
        }
        if (checkTypeUnique(deviceType)){
            throw new BusinessException("设备类型值不能重复");
        }
        this.updateById(deviceType);
        return true;
    }

    @Override
    public Boolean checkNameUnique(DeviceType deviceType) {
        Integer id =  null == deviceType.getId() ? -1 : deviceType.getId();
        LambdaQueryWrapper<DeviceType> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(DeviceType::getTypeName,deviceType.getTypeName())
                .eq(DeviceType::getEState,false);
        DeviceType one = this.getOne(queryWrapper);
        if (Objects.nonNull(one) && one.getId() != id.longValue())
        {
            return true;
        }
        return false;
    }

    public Boolean checkTypeUnique(DeviceType deviceType) {
        Integer id =  null == deviceType.getId() ? -1 : deviceType.getId();
        LambdaQueryWrapper<DeviceType> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(DeviceType::getTypeValue,deviceType.getTypeValue())
                .eq(DeviceType::getEState,false);
        DeviceType one = this.getOne(queryWrapper);
        if (Objects.nonNull(one) && one.getId() != id.longValue())
        {
            return true;
        }
        return false;
    }

    @Override
    public CommonPage<DeviceType> list(Integer current, Integer size, String name, String typeValue) {
        IPage<DeviceType> page = new Page<>(current,size);
        page = baseMapper.page(page, name,typeValue);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getSize(),page.getCurrent());
    }

    @Override
    public String export(String name, String typeValue) {
        ExportParams params = new ExportParams(null, "设备类型");
        return ExportExcelUtil.export("设备类型", param -> ExcelExportUtil.exportBigExcel(params, DeviceTypeExport.class,
                (o, i) -> {
                    CommonPage<DeviceType> page = this.list(i,30,name,typeValue);
                    page.getRecords().forEach(this::enhanceData);
                    return new ArrayList<>(BeanMapperUtils.mapList(page.getRecords(), DeviceType.class, DeviceTypeExport.class));
                }, null));
    }

    @Override
    public List<DeviceType> all(String name, String typeValue) {
        LambdaQueryWrapper<DeviceType> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper
                .select(DeviceType::getId,DeviceType::getTypeName)
                .like(StringUtils.isNotBlank(name), DeviceType::getTypeName, name)
                .like(StringUtils.isNotBlank(typeValue), DeviceType::getTypeValue, typeValue);
        return this.list(queryWrapper);
    }

    public void enhanceData(DeviceType deviceType){
        if (1 == deviceType.getTransmissionModel()){
            deviceType.setTm("NB-IOT");
        }else if (2 == deviceType.getTransmissionModel()){
            deviceType.setTm("Lord-IOT");
        }else {
            deviceType.setTm("2G/4G/5G");
        }
    }

    @Override
    public boolean remove(Integer id) {
        DeviceType deviceTypeById = this.getById(id);
        LambdaQueryWrapper<Device> deviceLambdaQueryWrapper = Wrappers.lambdaQuery();
        deviceLambdaQueryWrapper.eq(Device::getDwtype,deviceTypeById.getTypeValue());
        int count = deviceService.count(deviceLambdaQueryWrapper);
        if (count > 0){
            throw new BusinessException("存在关联设备不能删除");
        }
        DeviceType deviceType= new DeviceType();
        deviceType.setId(id);
        deviceType.setEState(true);
        return this.updateById(deviceType);
    }
}
