package com.usky.dm.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.dm.domain.DevopsTeam;
import com.usky.dm.domain.DevopsTeamRelation;
import com.usky.dm.mapper.DevopsTeamRelationMapper;
import com.usky.dm.service.DevopsTeamRelationService;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * <p>
 * 团队-人员 服务实现类
 * </p>
 *
 * @author ya
 * @since 2022-02-22
 */
@Service
public class DevopsTeamRelationServiceImpl extends AbstractCrudService<DevopsTeamRelationMapper, DevopsTeamRelation> implements DevopsTeamRelationService {

    @Override
    public void removeByTeamId(Integer teamId) {
        LambdaQueryWrapper<DevopsTeamRelation> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(DevopsTeamRelation::getFid,teamId);
        this.remove(queryWrapper);
    }

    @Override
    public void insertList(DevopsTeam devopsTeam) {
        devopsTeam.getUserIds().forEach(d -> d.setFid(devopsTeam.getId()));
        if (CollectionUtils.isEmpty(devopsTeam.getUserIds())){
            throw new BusinessException("用户信息不能为空");
        }
        if (devopsTeam.getUserIds().stream().noneMatch(d -> d.getLabel() == 2)){
            throw new BusinessException("责任人不能为空");
        }
        this.saveBatch(devopsTeam.getUserIds());
    }

    @Override
    public List<DevopsTeamRelation> list(Integer teamId) {
        LambdaQueryWrapper<DevopsTeamRelation> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(DevopsTeamRelation::getFid,teamId);
        return this.list(queryWrapper);
    }

    @Override
    public long getGroupByPhone() {
        LambdaQueryWrapper<DevopsTeamRelation> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.select(DevopsTeamRelation::getPhone)
                .groupBy(DevopsTeamRelation::getPhone);
        return this.list(queryWrapper).size();
    }
}
