package com.usky.dm.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.dm.domain.DevopsTeam;
import com.usky.dm.mapper.DevopsTeamMapper;
import com.usky.dm.service.DevopsTeamRelationService;
import com.usky.dm.service.DevopsTeamService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.Objects;

/**
 * <p>
 * 班组人员 服务实现类
 * </p>
 *
 * @author ya
 * @since 2022-02-22
 */
@Service
public class DevopsTeamServiceImpl extends AbstractCrudService<DevopsTeamMapper, DevopsTeam> implements DevopsTeamService {

    @Autowired
    private DevopsTeamRelationService devopsTeamRelationService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean add(DevopsTeam devopsTeam) {
        devopsTeam.setAddPeople(SecurityUtils.getUsername());
        devopsTeam.setAddTime(new Date());
        if (checkNameUnique(devopsTeam)){
            throw new BusinessException("班组名称不能重复");
        }
        this.save(devopsTeam);
        devopsTeamRelationService.insertList(devopsTeam);
        return true;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(DevopsTeam devopsTeam) {
        if (checkNameUnique(devopsTeam)){
            throw new BusinessException("班组名称不能重复");
        }
        this.updateById(devopsTeam);
        devopsTeamRelationService.removeByTeamId(devopsTeam.getId());
        devopsTeamRelationService.insertList(devopsTeam);
        return true;
    }

    @Override
    public Boolean checkNameUnique(DevopsTeam devopsTeam) {
        Integer id =  null == devopsTeam.getId() ? -1 : devopsTeam.getId();
        LambdaQueryWrapper<DevopsTeam> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(DevopsTeam::getTeamName,devopsTeam.getTeamName())
                .eq(DevopsTeam::getEState,1);
        DevopsTeam one = this.getOne(queryWrapper);
        if (Objects.nonNull(one) && one.getId() != id.longValue())
        {
            return true;
        }
        return false;
    }

    @Override
    public CommonPage<DevopsTeam> list(Integer current, Integer size, String name, String userName) {
        IPage<DevopsTeam> page = new Page<>(current,size);
        page = baseMapper.page(page,name,userName);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getSize(),page.getCurrent());
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Integer id) {
        DevopsTeam devopsTeam= new DevopsTeam();
        devopsTeam.setId(id);
        devopsTeam.setEState(0);
        devopsTeamRelationService.removeByTeamId(id);
        return this.updateById(devopsTeam);
    }
}
