package com.usky.dm.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.core.util.BeanMapperUtils;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.dm.domain.EventManage;
import com.usky.dm.mapper.EventManageMapper;
import com.usky.dm.service.EventManageService;
import com.usky.dm.service.util.Arith;
import com.usky.dm.service.util.ExportExcelUtil;
import com.usky.dm.service.vo.AlertCheckVO;
import com.usky.dm.service.vo.EventManageExport;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * <p>
 * 事件管理 服务实现类
 * </p>
 *
 * @author ya
 * @since 2022-02-22
 */
@Service
public class EventManageServiceImpl extends AbstractCrudService<EventManageMapper, EventManage> implements EventManageService {

    @Override
    public boolean add(EventManage eventManage) {
        eventManage.setReportingTime(new Date());
        if (checkNameUnique(eventManage)){
            throw new BusinessException("事件名称不能重复");
        }
        this.save(eventManage);
        return true;
    }

    @Override
    public boolean update(EventManage eventManage) {
        eventManage.setClTiem(new Date());
        eventManage.setClPeople(SecurityUtils.getUsername());
        if (checkNameUnique(eventManage)){
            throw new BusinessException("事件名称不能重复");
        }
        this.updateById(eventManage);
        return true;
    }

    @Override
    public Boolean checkNameUnique(EventManage eventManage) {
        Integer id =  null == eventManage.getId() ? -1 : eventManage.getId();
        LambdaQueryWrapper<EventManage> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(EventManage::getEventName,eventManage.getEventName());
        EventManage one = this.getOne(queryWrapper);
        if (Objects.nonNull(one) && one.getId() != id.longValue())
        {
            return true;
        }
        return false;
    }

    @Override
    public CommonPage<EventManage> list(Integer current, Integer size, String name, String type, Date startTime,Date endTime) {
        IPage<EventManage> page = new Page<>(current,size);
        page = baseMapper.page(page, name,type,startTime,endTime);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getSize(),page.getCurrent());
    }

    @Override
    public String export(String name, String type, Date startTime,Date endTime) {
        ExportParams params = new ExportParams(null, "事件");
        return ExportExcelUtil.export("事件", param -> ExcelExportUtil.exportBigExcel(params, EventManageExport.class,
                (o, i) -> {
                    CommonPage<EventManage> page = this.list(i,30,name,type,startTime,endTime);
                    return new ArrayList<>(BeanMapperUtils.mapList(page.getRecords(), EventManage.class, EventManageExport.class));
                }, null));
    }

    @Override
    public AlertCheckVO getCollect(Integer type) {
        QueryWrapper<EventManage> queryWrapper = new QueryWrapper<>();
        queryWrapper.select("clzt","count(*) as count")
                .eq(null != type && 0 != type,"eventType",type)
                .groupBy("clzt");
        List<Map<String, Object>> list = this.listMaps(queryWrapper);
        AlertCheckVO alertCheckVO = new AlertCheckVO();
        Integer noCheck = list.stream().filter(map -> "0".equals(map.get("clzt").toString())).findAny().map(map -> Integer.parseInt(map.get("count").toString())).orElse(0);
        Integer check = list.stream().filter(map -> "1".equals(map.get("clzt").toString())).findAny().map(map -> Integer.parseInt(map.get("count").toString())).orElse(0);
        alertCheckVO.setCheckCount(check);
        alertCheckVO.setNoCheckCount(noCheck);
        alertCheckVO.setTotal(check+noCheck);
        alertCheckVO.setCheckRadio(0 == alertCheckVO.getTotal() ? 0 : Arith.div(check,alertCheckVO.getTotal()));
        return alertCheckVO;
    }

    @Override
    public boolean remove(Integer id) {
        return false;
    }
}
