package com.usky.dm.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.core.util.BeanMapperUtils;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.dm.domain.Facility;
import com.usky.dm.mapper.FacilityMapper;
import com.usky.dm.service.FacilityService;
import com.usky.dm.service.util.ExportExcelUtil;
import com.usky.dm.service.vo.FacilityExport;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Objects;

/**
 * <p>
 * 资产表 服务实现类
 * </p>
 *
 * @author ya
 * @since 2022-02-22
 */
@Service
public class FacilityServiceImpl extends AbstractCrudService<FacilityMapper, Facility> implements FacilityService {

    @Override
    public boolean add(Facility facility) {
        if (checkNameUnique(facility)){
            throw new BusinessException("资产名称已存在");
        }
        this.save(facility);
        return true;
    }

    @Override
    public boolean update(Facility facility) {
        if (checkNameUnique(facility)){
            throw new BusinessException("资产名称已存在");
        }
        this.updateById(facility);
        return true;
    }

    @Override
    public Boolean checkNameUnique(Facility facility) {
        Integer id =  null == facility.getId() ? -1 : facility.getId();
        LambdaQueryWrapper<Facility> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(Facility::getFacilityName,facility.getFacilityName())
                .eq(Facility::getStatus,0);
        Facility one = this.getOne(queryWrapper);
        if (Objects.nonNull(one) && one.getId() != id.longValue())
        {
            return true;
        }
        return false;
    }

    @Override
    public CommonPage<Facility> list(Integer current, Integer size, String name, String address) {
        IPage<Facility> page = new Page<>(current, size);
        page = baseMapper.page(page, name, address);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getSize(),page.getCurrent());
    }


    @Override
    public String export(String name, String address) {
        ExportParams params = new ExportParams(null, "资产信息");
        return ExportExcelUtil.export("资产信息", param -> ExcelExportUtil.exportBigExcel(params, FacilityExport.class,
                (o, i) -> {
                    CommonPage<Facility> page = this.list(i,30,name,address);
                    return new ArrayList<>(BeanMapperUtils.mapList(page.getRecords(), Facility.class, FacilityExport.class));
                }, null));
    }

    @Override
    public boolean remove(Integer id) {
        Facility facility= new Facility();
        facility.setId(id);
        facility.setStatus(1);
        return this.updateById(facility);
    }
}
