package com.usky.dm.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.dm.domain.Floor;
import com.usky.dm.domain.Room;
import com.usky.dm.mapper.FloorMapper;
import com.usky.dm.service.FloorService;
import com.usky.dm.service.RoomService;;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Objects;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author ya
 * @since 2022-02-22
 */
@Service
public class FloorServiceImpl extends AbstractCrudService<FloorMapper, Floor> implements FloorService {

    @Autowired
    private RoomService roomService;

    @Override
    public Boolean add(Floor floor) {
        floor.setCreatMan(SecurityUtils.getUsername());
        if (checkNameUnique(floor)){
            throw new BusinessException("楼层名称不能重复");
        }
        this.save(floor);
        return true;
    }

    @Override
    public Boolean update(Floor floor) {
        if (checkNameUnique(floor)){
            throw new BusinessException("楼层名称不能重复");
        }
        this.updateById(floor);
        return true;
    }


    @Override
    public Boolean checkNameUnique(Floor floor) {
        Long id =  null == floor.getId() ? -1 : floor.getId();
        LambdaQueryWrapper<Floor> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(Floor::getFloorName,floor.getFloorName())
                .eq(Floor::getStatus,0)
                .eq(Floor::getBuildId,floor.getBuildId());
        Floor one = this.getOne(queryWrapper);
        if (Objects.nonNull(one) && one.getId() != id.longValue())
        {
            return true;
        }
        return false;
    }

    @Override
    public Boolean delete(Long id) {
        LambdaQueryWrapper<Room> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(Room::getStatus,0)
        .eq(Room::getFloorId,id);
        int count = roomService.count(queryWrapper);
        if (count > 0){
            throw new BusinessException("请先删除下属房间信息");
        }
        Floor floor = new Floor();
        floor.setId(id);
        floor.setStatus(1);
        return this.updateById(floor);
    }

    @Override
    public CommonPage<Floor> page(String name,String buildName, Long buildId, Integer current, Integer size) {
        IPage<Floor> page = new Page<>(current, size);
        page = baseMapper.page(page,name,buildName,buildId);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getSize(),page.getCurrent());
    }
}
