package com.usky.dm.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.core.util.BeanMapperUtils;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.dm.domain.InspectionPoint;
import com.usky.dm.mapper.InspectionPointMapper;
import com.usky.dm.service.InspectionPointService;
import com.usky.dm.service.util.ExportExcelUtil;
import com.usky.dm.service.vo.InspectionPointExport;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Objects;

/**
 * <p>
 * 维保点位 服务实现类
 * </p>
 *
 * @author ya
 * @since 2022-02-22
 */
@Service
public class InspectionPointServiceImpl extends AbstractCrudService<InspectionPointMapper, InspectionPoint> implements InspectionPointService {


    @Override
    public boolean add(InspectionPoint inspectionPoint) {
        if (checkNameUnique(inspectionPoint)){
            throw new BusinessException("点位名称不能重复");
        }
        this.save(inspectionPoint);
        return true;
    }

    @Override
    public boolean update(InspectionPoint inspectionPoint) {
        if (checkNameUnique(inspectionPoint)){
            throw new BusinessException("点位名称不能重复");
        }
        this.updateById(inspectionPoint);
        return true;
    }

    @Override
    public Boolean checkNameUnique(InspectionPoint inspectionPoint) {
        Integer id =  null == inspectionPoint.getId() ? -1 : inspectionPoint.getId();
        LambdaQueryWrapper<InspectionPoint> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(InspectionPoint::getSpotName,inspectionPoint.getSpotName())
                .eq(InspectionPoint::getDelFlag,true);
        InspectionPoint one = this.getOne(queryWrapper);
        if (Objects.nonNull(one) && one.getId() != id.longValue())
        {
            return true;
        }
        return false;
    }

    @Override
    public CommonPage<InspectionPoint> list(Integer current, Integer size, String name,String address) {
        IPage<InspectionPoint> page = new Page<>(current,size);
        page = baseMapper.page(page,name,address);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getSize(),page.getCurrent());
    }

    @Override
    public String export(String name, String address) {
        ExportParams params = new ExportParams(null, "点位信息");
        return ExportExcelUtil.export("点位信息", param -> ExcelExportUtil.exportBigExcel(params, InspectionPointExport.class,
                (o, i) -> {
                    CommonPage<InspectionPoint> page = this.list(i,30,name,address);
                    return new ArrayList<>(BeanMapperUtils.mapList(page.getRecords(), InspectionPoint.class, InspectionPointExport.class));
                }, null));
    }

    @Override
    public boolean remove(Integer id) {
        InspectionPoint inspectionPoint= new InspectionPoint();
        inspectionPoint.setId(id);
        inspectionPoint.setDelFlag(false);
        return this.updateById(inspectionPoint);
    }
}
