package com.usky.dm.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.util.StringUtils;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.dm.domain.*;
import com.usky.dm.mapper.MaintianLogMapper;
import com.usky.dm.mapper.vo.MaintianLogRequest;
import com.usky.dm.service.*;
import com.usky.dm.service.enums.WorkOrderType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 维保记录 服务实现类
 * </p>
 *
 * @author ya
 * @since 2022-02-22
 */
@Service
public class MaintianLogServiceImpl extends AbstractCrudService<MaintianLogMapper, MaintianLog> implements MaintianLogService {

    @Autowired
    private PlanChildService planChildService;

    @Autowired
    private PlanSpotService planSpotService;
    @Autowired
    private PlanPathService planPathService;
    @Autowired
    private WorkOrderService workOrderService;
    @Autowired
    private DevopsTeamService devopsTeamService;

    @Override
    public boolean add(MaintianLog maintianLog) {
        this.save(maintianLog);
        return true;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean addBatch(Integer planId, PlanChild planChild) {
        DevopsTeam devopsTeam = devopsTeamService.getById(planChild.getTeamId());
        List<PlanSpot> list = planSpotService.list(planChild.getPlanPathId().intValue());
        List<MaintianLog> maintianLogs = new ArrayList<>();
        for (PlanSpot planSpot:list) {
            MaintianLog maintianLog = new MaintianLog();
            maintianLog.setZbId(planChild.getId());
            maintianLog.setSpotId(planSpot.getSpotId());
            maintianLog.setPeopleId(0);
            maintianLog.setSpotAbnormal(4);
            maintianLog.setYwcTime(planChild.getEndTime());
            maintianLog.setCompanyCode(planChild.getCompanyCode());
            maintianLog.setData1(planChild.getData1());
            maintianLog.setData2("2");
            maintianLog.setData3("1");
            maintianLogs.add(maintianLog);
        }
        this.saveBatch(maintianLogs);
        List<WorkOrder> workOrderList = new ArrayList<>();
        for (MaintianLog maintianLog:maintianLogs) {
            workOrderList.add(workOrderService.generateWorkerOrder(workOrder -> {
                workOrder.setTeamId(planChild.getTeamId());
                workOrder.setTeamName(devopsTeam.getTeamName());
                workOrder.setAddPeople(maintianLog.getWbUserName());
                workOrder.setWorkOrderName(WorkOrderType.PLAN_DETAIL.getName());
                workOrder.setWorkOrderContent(WorkOrderType.PLAN_DETAIL.getName());
                workOrder.setWorkOrderType(WorkOrderType.PLAN_DETAIL.getCode());
                workOrder.setBusinessId(maintianLog.getId().toString());
                return workOrder;
            }));
        }
        return workOrderService.saveBatch(workOrderList);
    }

    @Override
    public boolean update(MaintianLog maintianLog) {
        maintianLog.setWcTime(new Date());
        this.updateById(maintianLog);
        LambdaQueryWrapper<MaintianLog> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(MaintianLog::getZbId,maintianLog.getZbId())
                .eq(MaintianLog::getIsComplete,false);
        int count = this.count(queryWrapper);
        if (count == 0){
            PlanChild planChild = new PlanChild();
            planChild.setId(maintianLog.getZbId());
            planChild.setIsComplete(true);
            planChildService.updateById(planChild);
        }
        return false;
    }

    @Override
    public Boolean checkNameUnique(MaintianLog maintianLog) {
        return null;
    }

    @Override
    public CommonPage<MaintianLog> list(MaintianLogRequest maintianLogRequest) {
        IPage<MaintianLog> page = new Page<>(maintianLogRequest.getCurrent(),maintianLogRequest.getSize());
        page = baseMapper.page(page,maintianLogRequest);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getSize(),page.getCurrent());
    }

    @Override
    public String export(String name, Integer dutyId) {
        return null;
    }


    @Override
    public boolean remove(Integer id) {
        return false;
    }

    @Override
    public void checkWorkOrder(WorkOrder workOrder) {
        workOrder.setWorkOrderType(WorkOrderType.PLAN_DETAIL.getCode());
        workOrderService.checkWorkOrder(workOrder,wo -> {
            if (StringUtils.isNotBlank(wo.getBusinessId())){
                MaintianLog maintianLog = new MaintianLog();
                maintianLog.setId(Integer.parseInt(wo.getBusinessId()));
                maintianLog.setIsComplete(true);
                return this.updateById(maintianLog);
            }else {
                return null;
            }
        });
    }
}
