package com.usky.dm.service.impl;


import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.core.util.BeanMapperUtils;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.dm.domain.MaintianPlan;
import com.usky.dm.domain.PlanChild;
import com.usky.dm.mapper.MaintianPlanMapper;
import com.usky.dm.service.MaintianLogService;
import com.usky.dm.service.MaintianPlanService;
import com.usky.dm.service.PlanChildService;
import com.usky.dm.service.util.ExportExcelUtil;
import com.usky.dm.service.vo.MaintianPlanExport;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Objects;

/**
 * <p>
 * 维保计划 服务实现类
 * </p>
 *
 * @author ya
 * @since 2022-02-22
 */
@Service
public class MaintianPlanServiceImpl extends AbstractCrudService<MaintianPlanMapper, MaintianPlan> implements MaintianPlanService {



    @Autowired
    private PlanChildService planChildService;
    @Autowired
    private MaintianLogService maintianLogService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean add(MaintianPlan maintianPlan) {
        maintianPlan.setData1(SecurityUtils.getUsername());
        maintianPlan.setData3("0");
        if (checkNameUnique(maintianPlan)){
            throw new BusinessException("计划名称不能重复");
        }
        this.save(maintianPlan);
        PlanChild planChild = planChildService.add(maintianPlan);
        maintianLogService.addBatch(maintianPlan.getId(),planChild);
        return true;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(MaintianPlan maintianPlan) {
        if (checkNameUnique(maintianPlan)){
            throw new BusinessException("计划名称不能重复");
        }
        this.updateById(maintianPlan);
        return true;
    }

    @Override
    public Boolean checkNameUnique(MaintianPlan maintianPlan) {
        Integer id =  null == maintianPlan.getId() ? -1 : maintianPlan.getId();
        LambdaQueryWrapper<MaintianPlan> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(MaintianPlan::getPlanName,maintianPlan.getPlanName())
                .eq(MaintianPlan::getData3,"0");
        MaintianPlan one = this.getOne(queryWrapper);
        if (Objects.nonNull(one) && one.getId() != id.longValue())
        {
            return true;
        }
        return false;
    }

    @Override
    public CommonPage<MaintianPlan> list(Integer current, Integer size, String name, Integer dutyId) {
        IPage<MaintianPlan> page = new Page<>(current,size);
        page = baseMapper.page(page,name,dutyId);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getSize(),page.getCurrent());
    }

    @Override
    public String export(String name, Integer dutyId) {
        ExportParams params = new ExportParams(null, "巡检计划");
        return ExportExcelUtil.export("巡检计划", param -> ExcelExportUtil.exportBigExcel(params, MaintianPlanExport.class,
                (o, i) -> {
                    CommonPage<MaintianPlan> page = this.list(i,30,name,dutyId);
                    return new ArrayList<>(BeanMapperUtils.mapList(page.getRecords(), MaintianPlan.class, MaintianPlanExport.class));
                }, null));
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Integer id) {
        MaintianPlan maintianPlan= new MaintianPlan();
        maintianPlan.setId(id);
        maintianPlan.setData3("1");
        return this.updateById(maintianPlan);
    }
}
