package com.usky.dm.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.core.util.BeanMapperUtils;
import com.usky.common.core.util.StringUtils;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.dm.domain.MonitorPoint;
import com.usky.dm.mapper.MonitorPointMapper;
import com.usky.dm.service.MonitorDeviceService;
import com.usky.dm.service.MonitorPointService;
import com.usky.dm.service.util.ExportExcelUtil;
import com.usky.dm.service.vo.MonitorPointExport;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.Objects;

/**
 * <p>
 * 监测点位 服务实现类
 * </p>
 *
 * @author ya
 * @since 2022-02-22
 */
@Service
public class MonitorPointServiceImpl extends AbstractCrudService<MonitorPointMapper, MonitorPoint> implements MonitorPointService {

    @Autowired
    private MonitorDeviceService monitorDeviceService;


    @Override
    public boolean add(MonitorPoint monitorPoint) {
        monitorPoint.setAddPeople(SecurityUtils.getUsername());
        if (checkNameUnique(monitorPoint)){
            throw new BusinessException("监测名称不能重复");
        }
        this.save(monitorPoint);
        monitorDeviceService.insertList(monitorPoint);
        return true;
    }

    @Override
    public boolean update(MonitorPoint monitorPoint) {
        if (checkNameUnique(monitorPoint)){
            throw new BusinessException("监测名称不能重复");
        }
        this.updateById(monitorPoint);
        monitorDeviceService.removeByTeamId(monitorPoint.getId());
        monitorDeviceService.insertList(monitorPoint);
        return true;
    }

    @Override
    public Boolean checkNameUnique(MonitorPoint monitorPoint) {
        Integer id =  null == monitorPoint.getId() ? -1 : monitorPoint.getId();
        LambdaQueryWrapper<MonitorPoint> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(MonitorPoint::getObjectName,monitorPoint.getObjectName())
                .eq(MonitorPoint::getEState,0);
        MonitorPoint one = this.getOne(queryWrapper);
        if (Objects.nonNull(one) && one.getId() != id.longValue())
        {
            return true;
        }
        return false;
    }

    @Override
    public CommonPage<MonitorPoint> list(Integer current, Integer size, String name,Date startTime, Date endTime) {
        IPage<MonitorPoint> page = new Page<>(current,size);
        LambdaQueryWrapper<MonitorPoint> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper
                .like(StringUtils.isNotBlank(name),MonitorPoint::getObjectName,name)
                .between(null != startTime && null != endTime,MonitorPoint::getAddTime,startTime,endTime)
                .eq(MonitorPoint::getEState,0)
                .orderByDesc(MonitorPoint::getId);
        page = this.page(page, queryWrapper);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getSize(),page.getCurrent());
    }

    @Override
    public String export(String name, Date startTime, Date endTime) {
        ExportParams params = new ExportParams(null, "监测点位");
        return ExportExcelUtil.export("监测点位", param -> ExcelExportUtil.exportBigExcel(params, MonitorPointExport.class,
                (o, i) -> {
                    CommonPage<MonitorPoint> page = this.list(i,30,name,startTime,endTime);
                    return new ArrayList<>(BeanMapperUtils.mapList(page.getRecords(), MonitorPoint.class, MonitorPointExport.class));
                }, null));
    }

    @Override
    public boolean remove(Integer id) {
        monitorDeviceService.removeByTeamId(id);
        MonitorPoint monitorPoint= new MonitorPoint();
        monitorPoint.setId(id);
        monitorPoint.setEState(1);
        return this.updateById(monitorPoint);
    }
}
