package com.usky.dm.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.util.BeanMapperUtils;
import com.usky.common.core.util.DateUtils;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.dm.domain.MaintianPlan;
import com.usky.dm.domain.PlanChild;
import com.usky.dm.mapper.PlanChildMapper;
import com.usky.dm.mapper.vo.MaintianLogRequest;
import com.usky.dm.service.DevopsTeamRelationService;
import com.usky.dm.service.PlanChildService;
import com.usky.dm.service.util.ExportExcelUtil;
import com.usky.dm.service.vo.PlanChildExport;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalTime;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 * 维保计划关联关系表 服务实现类
 * </p>
 *
 * @author ya
 * @since 2022-02-22
 */
@Service
public class PlanChildServiceImpl extends AbstractCrudService<PlanChildMapper, PlanChild> implements PlanChildService {

    @Autowired
    private DevopsTeamRelationService devopsTeamRelationService;

    @Override
    public CommonPage<PlanChild> list(MaintianLogRequest maintianLogRequest) {
        IPage<PlanChild> page = new Page<>(maintianLogRequest.getCurrent(),maintianLogRequest.getSize());
        page = baseMapper.page(page,maintianLogRequest);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getSize(),page.getCurrent());
    }

    @Override
    public Map<String, Object> getCollect() {
        Date date = new Date();
        Map<String,Object> map = new HashMap<>();
        map.put("userCount",devopsTeamRelationService.getGroupByPhone());
        LambdaQueryWrapper<PlanChild> totalQuery = Wrappers.lambdaQuery();
        totalQuery.between(PlanChild::getInspectDate, DateUtils.getDayStartTime(date),DateUtils.getDayEndTime(date));
        map.put("todayCount",this.count(totalQuery));
        LambdaQueryWrapper<PlanChild> totalTimeOutQuery = Wrappers.lambdaQuery();
        totalTimeOutQuery.between(PlanChild::getInspectDate, DateUtils.getDayStartTime(date),DateUtils.getDayEndTime(date))
                .eq(PlanChild::getIsComplete,false)
                .lt(PlanChild::getEndTime,date);
        map.put("todayTimeOutCount",this.count(totalTimeOutQuery));
        LambdaQueryWrapper<PlanChild> totalCompleteQuery = Wrappers.lambdaQuery();
        totalCompleteQuery.between(PlanChild::getInspectDate, DateUtils.getDayStartTime(date),DateUtils.getDayEndTime(date))
                .eq(PlanChild::getIsComplete,true);
        map.put("todayCompleteCount",this.count(totalCompleteQuery));
        return map;
    }

    @Override
    public String export(MaintianLogRequest maintianLogRequest) {
        ExportParams params = new ExportParams(null, "巡检任务");
        return ExportExcelUtil.export("巡检任务", param -> ExcelExportUtil.exportBigExcel(params, PlanChildExport.class,
                (o, i) -> {
                    maintianLogRequest.setCurrent(i);
                    maintianLogRequest.setSize(30);
                    CommonPage<PlanChild> page = this.list(maintianLogRequest);
                    return new ArrayList<>(BeanMapperUtils.mapList(page.getRecords(), PlanChild.class, PlanChildExport.class));
                }, null));
    }

    @Override
    public PlanChild add(MaintianPlan maintianPlan) {
        PlanChild planChild = new PlanChild();
        planChild.setPlanPathId(maintianPlan.getPlanPathId());
        planChild.setZbId(maintianPlan.getId());
        planChild.setTeamId(maintianPlan.getTeamId());
        planChild.setSpotId(11111111);
        planChild.setInspectDate(new Date());
        planChild.setStartTime(getTime(planChild.getInspectDate(),maintianPlan.getStartInspectTime()));
        planChild.setEndTime(getTime(planChild.getInspectDate(),maintianPlan.getEndInspectTime()));
        planChild.setCompanyCode(maintianPlan.getCompanyCode());
        planChild.setCompletion("0");
        planChild.setData1(maintianPlan.getData1());
        planChild.setData2("2");
        this.save(planChild);
        return planChild;
    }

    public Date getTime(Date date, LocalTime localTime){
        Date dayStartTime = DateUtils.getDayStartTime(date);
        dayStartTime = DateUtils.addHours(dayStartTime,localTime.getHour());
        dayStartTime = DateUtils.addMinutes(dayStartTime,localTime.getMinute());
        dayStartTime = DateUtils.addSeconds(dayStartTime,localTime.getSecond());
        return dayStartTime;
    }
}
