package com.usky.dm.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.dm.domain.PlanPath;
import com.usky.dm.mapper.PlanPathMapper;
import com.usky.dm.service.PlanPathService;
import com.usky.dm.service.PlanSpotService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.Objects;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author ya
 * @since 2022-04-25
 */
@Service
public class PlanPathServiceImpl extends AbstractCrudService<PlanPathMapper, PlanPath> implements PlanPathService {

    @Autowired
    private PlanSpotService planSpotService;

    @Override
    public boolean add(PlanPath planPath) {
        planPath.setCreateBy(SecurityUtils.getUsername());
        planPath.setCreateTime(new Date());
        if (checkNameUnique(planPath)){
            throw new BusinessException("线路名称不能重复");
        }
        this.save(planPath);
        planSpotService.insertList(planPath);
        return true;
    }

    @Override
    public boolean update(PlanPath planPath) {
        if (checkNameUnique(planPath)){
            throw new BusinessException("线路名称不能重复");
        }
        this.updateById(planPath);
        planSpotService.removeByPlanId(planPath.getId().intValue());
        planSpotService.insertList(planPath);
        return true;
    }

    @Override
    public Boolean checkNameUnique(PlanPath planPath) {
        Long id =  null == planPath.getId() ? -1 : planPath.getId();
        LambdaQueryWrapper<PlanPath> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(PlanPath::getName,planPath.getName())
                .eq(PlanPath::getDelFlag,false);
        PlanPath one = this.getOne(queryWrapper);
        if (Objects.nonNull(one) && one.getId() != id.longValue())
        {
            return true;
        }
        return false;
    }

    @Override
    public CommonPage<PlanPath> list(Integer current, Integer size, String name) {
        IPage<PlanPath> page = new Page<>(current,size);
        LambdaQueryWrapper<PlanPath> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.like(StringUtils.isNotBlank(name),PlanPath::getName,name)
                .eq(PlanPath::getDelFlag,false)
                .orderByDesc(PlanPath::getId);
        page = this.page(page,queryWrapper);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getSize(),page.getCurrent());
    }

    @Override
    public boolean remove(Long id) {
        PlanPath planPath = new PlanPath();
        planPath.setId(id);
        planPath.setDelFlag(true);
        this.updateById(planPath);
        planSpotService.removeByPlanId(id.intValue());
        return true;
    }
}
