package com.usky.dm.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.core.util.StringUtils;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.dm.domain.Product;
import com.usky.dm.mapper.ProductMapper;
import com.usky.dm.service.ProductService;
import org.springframework.stereotype.Service;

import java.util.Objects;
import java.util.Optional;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author ya
 * @since 2022-03-31
 */
@Service
public class ProductServiceImpl extends AbstractCrudService<ProductMapper, Product> implements ProductService {

    @Override
    public boolean add(Product product) {
        product.setCreateBy(SecurityUtils.getUsername());
        return this.save(product);
    }

    @Override
    public boolean update(Product product) {
        product.setUpdateBy(SecurityUtils.getUsername());
        if (this.checkNameUnique(product))
        {
            throw new BusinessException("修改产品信息'" + product.getName() + "'失败，产品信息已存在");
        }
        Product byId = this.getById(product.getId());
        return this.updateById(product);
    }

    @Override
    public Boolean checkNameUnique(Product product) {
        Long configId = null != product.getId() && 0 != product.getId() ? product.getId() : -1L;
        LambdaQueryWrapper<Product> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(Product::getName,product.getName())
                .eq(Product::getDelFlag,0);
        Product one = this.getOne(queryWrapper);
        if (Objects.nonNull(one) && one.getId() != configId.longValue())
        {
            return true;
        }
        return false;
    }

    @Override
    public CommonPage<Product> list(Integer current, Integer size, String name) {
        IPage<Product> page = new Page<>(current, size);
        LambdaQueryWrapper<Product> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.like(StringUtils.isNotBlank(name),Product::getName,name)
                .eq(Product::getDelFlag,0)
                .orderByDesc(Product::getId);
        page = this.page(page,queryWrapper);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getCurrent(),page.getSize());
    }

    @Override
    public boolean remove(Long id) {
        Product product = this.getById(id);
        Optional.ofNullable(product).orElseThrow(() -> new BusinessException("产品不存在"));
        product.setDelFlag(1);
        return this.updateById(product);
    }
}
