package com.usky.dm.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.dm.domain.Room;
import com.usky.dm.mapper.RoomMapper;
import com.usky.dm.service.RoomService;
import org.springframework.stereotype.Service;

import java.util.Objects;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author ya
 * @since 2022-02-22
 */
@Service
public class RoomServiceImpl extends AbstractCrudService<RoomMapper, Room> implements RoomService {

    @Override
    public Boolean add(Room room)
    {
        room.setCreatMan(SecurityUtils.getUsername());
        if (checkNameUnique(room)){
            throw new BusinessException("房间名称不能重复");
        }
        this.save(room);
        return true;
    }

    @Override
    public Boolean update(Room room)
    {
        if (checkNameUnique(room)){
            throw new BusinessException("房间名称不能重复");
        }
        this.updateById(room);
        return true;
    }


    @Override
    public Boolean checkNameUnique(Room room) {
        Long id =  null == room.getId() ? -1 : room.getId();
        LambdaQueryWrapper<Room> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(Room::getRoomName,room.getRoomName())
                .eq(Room::getStatus,0)
                .eq(Room::getFloorId,room.getFloorId());
        Room one = this.getOne(queryWrapper);
        if (Objects.nonNull(one) && one.getId() != id.longValue())
        {
            return true;
        }
        return false;
    }

    @Override
    public Boolean delete(Long id) {
        Room room = new Room();
        room.setId(id);
        room.setStatus(1);
        return this.updateById(room);
    }

    @Override
    public CommonPage<Room> page(String name, String floorName, Long floorId, Integer current, Integer size,String buildName) {
        IPage<Room> page = new Page<>(current, size);
        page = baseMapper.page(page,name,floorName,floorId,buildName);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getSize(),page.getCurrent());
    }
}
