package com.usky.dm.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.util.BeanMapperUtils;
import com.usky.common.core.util.DateUtils;
import com.usky.common.core.util.JsonUtils;
import com.usky.common.core.util.StringUtils;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.dm.domain.TbAlarm;
import com.usky.dm.domain.WorkOrder;
import com.usky.dm.mapper.TbAlarmMapper;
import com.usky.dm.mapper.vo.AlertRequest;
import com.usky.dm.mapper.vo.DeviceCollectVO;
import com.usky.dm.service.TbAlarmService;
import com.usky.dm.service.WorkOrderService;
import com.usky.dm.service.enums.DeviceCollectCode;
import com.usky.dm.service.enums.WorkOrderType;
import com.usky.dm.service.util.AlertCheckUtil;
import com.usky.dm.service.util.ExportExcelUtil;
import com.usky.dm.service.vo.AlertCheckVO;
import com.usky.dm.service.vo.TbAlertExport;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.fasterxml.jackson.core.type.TypeReference;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * <p>
 * 设备告警表 服务实现类
 * </p>
 *
 * @author ya
 * @since 2022-02-28
 */
@Service
public class TbAlarmServiceImpl extends AbstractCrudService<TbAlarmMapper, TbAlarm> implements TbAlarmService {

    public static final String DEFAULT_LOCAL_DATE_FMT = "yyyy-MM-dd";
    @Autowired
    private WorkOrderService workOrderService;

    @Override
    public AlertCheckVO selectCollect(AlertRequest alertRequest) {
        alertRequest.setDeviceTypes(AlertCheckUtil.getType(alertRequest.getBigDeviceType()));
        return AlertCheckUtil.check(getCollect(alertRequest));
    }

    @Override
    public String export(AlertRequest alertRequest) {
        ExportParams params = new ExportParams(null, "告警");
        return ExportExcelUtil.export("告警", param -> ExcelExportUtil.exportBigExcel(params, TbAlertExport.class,
                (o, i) -> {
                    alertRequest.setCurrent(i);
                    alertRequest.setSize(30);
                    CommonPage<TbAlarm> page = this.page(alertRequest);
                    return new ArrayList<>(BeanMapperUtils.mapList(page.getRecords(), TbAlarm.class, TbAlertExport.class));
                }, null));
    }

    @Override
    public CommonPage<TbAlarm> page(AlertRequest alertRequest) {
        alertRequest.setDeviceTypes(AlertCheckUtil.getType(alertRequest.getBigDeviceType()));
        Page<TbAlarm> page = new Page<>(alertRequest.getCurrent(), alertRequest.getSize());
        LambdaQueryWrapper<TbAlarm> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper
                .between(null != alertRequest.getStartTime() && null != alertRequest.getEndTime(),TbAlarm::getAlarmTime,alertRequest.getStartTime(),alertRequest.getEndTime())
                .eq(null != alertRequest.getStatus(),TbAlarm::getAlarmStatus,alertRequest.getStatus())
                .eq(StringUtils.isNotBlank(alertRequest.getCompanyCode()),TbAlarm::getCompanyCode,alertRequest.getCompanyCode())
                .eq(null != alertRequest.getDeviceType(),TbAlarm::getDeviceType,alertRequest.getDeviceType())
                .notIn(null != alertRequest.getType() && 1 == alertRequest.getType(),TbAlarm::getProperty,getProperty())
                .in(null != alertRequest.getType() && 2 == alertRequest.getType(),TbAlarm::getProperty,getProperty())
                .in(null != alertRequest.getBigDeviceType(),TbAlarm::getDeviceType,alertRequest.getDeviceTypes())
                .eq(null != alertRequest.getSerial(),TbAlarm::getSerial,alertRequest.getSerial())
                .orderByAsc(TbAlarm::getAlarmStatus)
                .orderByDesc(TbAlarm::getCreateTime);
        page = this.page(page,queryWrapper);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getSize(),page.getCurrent());
    }


    @Override
    public List<Map<String,Object>> getCollectIco(AlertRequest alertRequest) {
        alertRequest.setDeviceTypes(AlertCheckUtil.getType(alertRequest.getBigDeviceType()));
        List<DeviceCollectVO> list = getCollect(alertRequest);
        return perfectDate(list,alertRequest.getStartTime(),alertRequest.getEndTime(),dataList -> {
            List<DeviceCollectVO> collectVOS = new ArrayList<>();
            for (DeviceCollectCode deviceCollectCode:DeviceCollectCode.values()) {
                DeviceCollectVO dv = new DeviceCollectVO();
                dv.setDeviceTypeS(deviceCollectCode.getName());
                dv.setDeviceCount(AlertCheckUtil.getCountByType(dataList,deviceCollectCode));
                collectVOS.add(dv);
            }
            return collectVOS;
        });
    }

    @Override
    public List<Map<String, Object>> getCollectIcoDetail(AlertRequest alertRequest) {
        alertRequest.setDeviceTypes(AlertCheckUtil.getType(alertRequest.getBigDeviceType()));
        List<DeviceCollectVO> list = getCollect(alertRequest);
        return perfectDate(list,alertRequest.getStartTime(),alertRequest.getEndTime(),dataList -> {
                  Map<String,Object> map = new HashMap<>();
                  map.put("checkCount",dataList.stream().filter(deviceCollectVO -> 1 == deviceCollectVO.getClzt())
                          .mapToInt(DeviceCollectVO::getDeviceCount).sum());
                  map.put("noCheckCount",dataList.stream().filter(deviceCollectVO -> 0 == deviceCollectVO.getClzt())
                            .mapToInt(DeviceCollectVO::getDeviceCount).sum());
                    return map;
                });

    }

    @Override
    public void checkWorkOrder(WorkOrder workOrder) {
        workOrder.setWorkOrderType(WorkOrderType.FIRE_ALERT.getCode());
        workOrderService.checkWorkOrder(workOrder,wo -> {
            if (StringUtils.isNotBlank(wo.getBusinessId())){
                TbAlarm tbAlarm = new TbAlarm();
                tbAlarm.setId(Long.parseLong(wo.getBusinessId()));
                tbAlarm.setAlarmStatus(1);
                tbAlarm.setUpdateTime(new Date());
                if (StringUtils.isBlank(workOrder.getCzPeople())){
                    tbAlarm.setUpdateBy(SecurityUtils.getUsername());
                }
                return this.updateById(tbAlarm);
            }else {
                return null;
            }
        });
    }


    /**
     * 完善没有的数据
     * @param list
     * @param startTime
     * @param endTime
     */
    public List<Map<String,Object>> perfectDate(List<DeviceCollectVO> list, Date startTime,Date endTime,Function<List<DeviceCollectVO>,Object> function){
        List<Map<String,Object>> reList = new ArrayList<>();
        List<DeviceCollectVO> dateList;
        do {
            String dates = DateUtils.format(startTime, DEFAULT_LOCAL_DATE_FMT);
            Map<String,Object> map = new HashMap<>();
            dateList = list.stream().filter(dv -> dv.getDeviceTime().equals(dates)).collect(Collectors.toList());
            map.put("time",dates);
            map.put("list",function.apply(dateList));
            reList.add(map);
            startTime = DateUtils.addDays(startTime,1);
        }while ((startTime.getTime() - endTime.getTime()) <= 0);
        return reList;
    }

    public List<DeviceCollectVO> getCollect(AlertRequest alertRequest){
        alertRequest.setDeviceTypes(AlertCheckUtil.getType(alertRequest.getBigDeviceType()));
        QueryWrapper<TbAlarm> query = Wrappers.query();
        query.select("device_type as deviceType","work_id as workId","count(*) deviceCount","DATE_FORMAT(alarm_time, '%Y-%m-%d') as deviceTime")
                .between(null != alertRequest.getStartTime() && null != alertRequest.getEndTime(),"alarm_time",alertRequest.getStartTime(),alertRequest.getEndTime())
                .eq(null != alertRequest.getStatus(),"alarm_status",alertRequest.getStatus())
                .eq(StringUtils.isNotBlank(alertRequest.getCompanyCode()),"company_code",alertRequest.getCompanyCode())
                .eq(null != alertRequest.getDeviceType(),"device_type",alertRequest.getDeviceType())
                .in(null != alertRequest.getType() && 2 == alertRequest.getType(),"property",getProperty())
                .notIn(null != alertRequest.getType() && 1 == alertRequest.getType(),"property",getProperty())
                .in(null != alertRequest.getBigDeviceType(),"device_type",alertRequest.getDeviceTypes())
                .isNotNull("work_id")
                .groupBy("deviceType","workId","deviceTime");
        List<Map<String, Object>> maps = this.listMaps(query);
        List<DeviceCollectVO> collectVOS = JsonUtils.fromJson(JsonUtils.toJson(maps), new TypeReference<List<DeviceCollectVO>>() {
        });
        return enhanceWorkAndAlarm(collectVOS);
    }


    /**
     * 填充状态信息
     * @param list
     * @return
     */
    public List<DeviceCollectVO> enhanceWorkAndAlarm(List<DeviceCollectVO> list){
        if (CollectionUtils.isNotEmpty(list)){
            LambdaQueryWrapper<WorkOrder> queryWrapper = Wrappers.lambdaQuery();
            queryWrapper.select(WorkOrder::getId,WorkOrder::getClzt)
                    .in(WorkOrder::getId,list.stream().map(DeviceCollectVO::getWorkId).collect(Collectors.toList()));
            List<WorkOrder> workOrderList = workOrderService.list(queryWrapper);
            list = list.stream().peek(deviceCollectVO -> {
                WorkOrder wo = workOrderList
                        .stream()
                        .filter(workOrder -> workOrder.getId().equals(deviceCollectVO.getWorkId()))
                        .findAny()
                        .get();
                deviceCollectVO.setClzt(wo.getClzt());
            }).collect(Collectors.toList());
            list = collectDate(list);
        }
        return list;
    }

    public List<DeviceCollectVO> collectDate(List<DeviceCollectVO> list){
        Map<String, Long> countMap = list.stream()
                .collect(Collectors.groupingBy(o -> o.getClzt() + "_" + o.getDeviceTime() + "_" + o.getWorkId()+ "_" + o.getDeviceType(),
                        Collectors.summingLong(DeviceCollectVO::getDeviceCount)));
        return countMap.keySet().stream().map(key -> {
            String[] arr = key.split("_");
            String clzt = arr[0];
            String deviceTime = arr[1];
            String workId = arr[2];
            String deviceType = arr[3];
            DeviceCollectVO obj = new DeviceCollectVO();
            obj.setClzt(Integer.parseInt(clzt));
            obj.setDeviceTime(deviceTime);
            obj.setWorkId(Integer.parseInt(workId));
            obj.setDeviceType(Integer.parseInt(deviceType));
            obj.setDeviceCount(countMap.get(key).intValue());
            return obj;
        }).collect(Collectors.toList());

    }

    public List<String> getProperty(){
        List<String> list = new ArrayList<>();
        list.add("火警");
        list.add("故障");
        return list;
    }
}
