package com.usky.dm.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.util.BeanMapperUtils;
import com.usky.common.core.util.DateUtils;
import com.usky.common.core.util.JsonUtils;
import com.usky.common.core.util.StringUtils;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.dm.domain.TbDeviceStatus;
import com.usky.dm.mapper.TbDeviceStatusMapper;
import com.usky.dm.mapper.vo.AlertRequest;
import com.usky.dm.mapper.vo.DeviceCollectVO;
import com.usky.dm.service.TbDeviceStatusService;
import com.usky.dm.service.enums.DeviceCollectCode;
import com.usky.dm.service.util.AlertCheckUtil;
import com.usky.dm.service.util.ExportExcelUtil;
import com.usky.dm.service.vo.AlertCheckVO;
import com.usky.dm.service.vo.TbDeviceStatusExport;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;



/**
 * <p>
 * 设备心跳表 服务实现类
 * </p>
 *
 * @author ya
 * @since 2022-03-14
 */
@Service
public class TbDeviceStatusServiceImpl extends AbstractCrudService<TbDeviceStatusMapper, TbDeviceStatus> implements TbDeviceStatusService {

    public static final String DEFAULT_LOCAL_DATE_FMT = "yyyy-MM-dd";
    @Override
    public AlertCheckVO selectCollect(AlertRequest alertRequest) {
        alertRequest.setDeviceTypes(AlertCheckUtil.getType(alertRequest.getBigDeviceType()));
        return AlertCheckUtil.check(getCollect(alertRequest));
    }

    @Override
    public String export(AlertRequest alertRequest) {
        ExportParams params = new ExportParams(null, "离线/故障报告");
        return ExportExcelUtil.export("告警", param -> ExcelExportUtil.exportBigExcel(params, TbDeviceStatusExport.class,
                (o, i) -> {
                    alertRequest.setCurrent(i);
                    alertRequest.setSize(30);
                    CommonPage<TbDeviceStatus> page = this.page(alertRequest);
                    return new ArrayList<>(BeanMapperUtils.mapList(page.getRecords(), TbDeviceStatus.class, TbDeviceStatusExport.class));
                }, null));
    }

    @Override
    public CommonPage<TbDeviceStatus> page(AlertRequest alertRequest) {
        alertRequest.setDeviceTypes(AlertCheckUtil.getType(alertRequest.getBigDeviceType()));
        Page<TbDeviceStatus> page = new Page<>(alertRequest.getCurrent(), alertRequest.getSize());
        LambdaQueryWrapper<TbDeviceStatus> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper
                .between(null != alertRequest.getStartTime() && null != alertRequest.getEndTime(),TbDeviceStatus::getCreateTime,alertRequest.getStartTime(),alertRequest.getEndTime())
                .eq(null != alertRequest.getStatus(),TbDeviceStatus::getAlarmStatus,alertRequest.getStatus())
                .eq(null != alertRequest.getCheckStatus(),TbDeviceStatus::getCheckStatus,alertRequest.getCheckStatus())
                .eq(StringUtils.isNotBlank(alertRequest.getCompanyCode()),TbDeviceStatus::getCompanyCode,alertRequest.getCompanyCode())
                .eq(null != alertRequest.getDeviceType(),TbDeviceStatus::getDeviceType,alertRequest.getDeviceType())
                .in(null != alertRequest.getBigDeviceType(),TbDeviceStatus::getDeviceType,alertRequest.getDeviceTypes())
                .orderByDesc(TbDeviceStatus::getCreateTime);
        page = this.page(page,queryWrapper);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getSize(),page.getCurrent());
    }

    @Override
    public List<Map<String, Object>> getCollectIco(AlertRequest alertRequest) {
        alertRequest.setDeviceTypes(AlertCheckUtil.getType(alertRequest.getBigDeviceType()));
        List<DeviceCollectVO> list = getCollect(alertRequest);
        return perfectDate(list,alertRequest.getStartTime(),alertRequest.getEndTime(),dataList -> {
            List<DeviceCollectVO> collectVOS = new ArrayList<>();
            for (DeviceCollectCode deviceCollectCode:DeviceCollectCode.values()) {
                DeviceCollectVO dv = new DeviceCollectVO();
                dv.setDeviceTypeS(deviceCollectCode.getName());
                dv.setDeviceCount(AlertCheckUtil.getCountByType(dataList,deviceCollectCode));
                collectVOS.add(dv);
            }
            return collectVOS;
        });
    }

    @Override
    public List<Map<String, Object>> getCollectIcoDetail(AlertRequest alertRequest) {
        alertRequest.setDeviceTypes(AlertCheckUtil.getType(alertRequest.getBigDeviceType()));
        List<DeviceCollectVO> list = getCollect(alertRequest);
        return perfectDate(list,alertRequest.getStartTime(),alertRequest.getEndTime(),dataList -> {
            Map<String,Object> map = new HashMap<>();
            map.put("checkCount",dataList.stream().filter(deviceCollectVO -> 1 == deviceCollectVO.getClzt())
                    .mapToInt(DeviceCollectVO::getDeviceCount).sum());
            map.put("noCheckCount",dataList.stream().filter(deviceCollectVO -> 0 == deviceCollectVO.getClzt())
                    .mapToInt(DeviceCollectVO::getDeviceCount).sum());
            return map;
        });
    }


    public List<Map<String,Object>> perfectDate(List<DeviceCollectVO> list, Date startTime, Date endTime, Function<List<DeviceCollectVO>,Object> function){
        List<Map<String,Object>> reList = new ArrayList<>();
        List<DeviceCollectVO> dateList;
        do {
            String dates = DateUtils.format(startTime, DEFAULT_LOCAL_DATE_FMT);
            Map<String,Object> map = new HashMap<>();
            dateList = list.stream().filter(dv -> dv.getDeviceTime().equals(dates)).collect(Collectors.toList());
            map.put("time",dates);
            map.put("list",function.apply(dateList));
            reList.add(map);
            startTime = DateUtils.addDays(startTime,1);
        }while ((startTime.getTime() - endTime.getTime()) <= 0);
        return reList;
    }

    public List<DeviceCollectVO> getCollect(AlertRequest alertRequest){
        alertRequest.setDeviceTypes(AlertCheckUtil.getType(alertRequest.getBigDeviceType()));
        QueryWrapper<TbDeviceStatus> query = Wrappers.query();
        query.select("device_type as deviceType","count(*) deviceCount","alarm_status as clzt","DATE_FORMAT(alarm_time, '%Y-%m-%d') as deviceTime")
                .between(null != alertRequest.getStartTime() && null != alertRequest.getEndTime(),"alarm_time",alertRequest.getStartTime(),alertRequest.getEndTime())
                .eq(null != alertRequest.getStatus(),"alarm_status",alertRequest.getStatus())
                .eq(null != alertRequest.getCheckStatus(),"check_status",alertRequest.getCheckStatus())
                .eq(StringUtils.isNotBlank(alertRequest.getCompanyCode()),"company_code",alertRequest.getCompanyCode())
                .eq(null != alertRequest.getDeviceType(),"device_type",alertRequest.getDeviceType())
                .in(CollectionUtils.isNotEmpty(alertRequest.getDeviceTypes()),"device_type",alertRequest.getDeviceTypes())
                .groupBy("deviceType","clzt","deviceTime");
        List<Map<String, Object>> maps = this.listMaps(query);
        return JsonUtils.fromJson(JsonUtils.toJson(maps), new com.fasterxml.jackson.core.type.TypeReference<List<DeviceCollectVO>>() {
        });
    }
}
