package com.usky.dm.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.util.StringUtils;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.dm.domain.TbInfo;
import com.usky.dm.mapper.TbInfoMapper;
import com.usky.dm.mapper.vo.InfoRequest;
import com.usky.dm.service.TbInfoService;
import com.usky.dm.service.util.AlertCheckUtil;
import org.springframework.stereotype.Service;

/**
 * <p>
 * 设备心跳表 服务实现类
 * </p>
 *
 * @author ya
 * @since 2022-02-28
 */
@Service
public class TbInfoServiceImpl extends AbstractCrudService<TbInfoMapper, TbInfo> implements TbInfoService {

    @Override
    public CommonPage<TbInfo> list(InfoRequest infoRequest) {
        infoRequest.setDeviceTypes(AlertCheckUtil.getType(infoRequest.getBigDeviceType()));
        Page<TbInfo> page = new Page<>(infoRequest.getCurrent(), infoRequest.getSize());
        LambdaQueryWrapper<TbInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper
                .between(null != infoRequest.getStartTime() && null != infoRequest.getEndTime(),TbInfo::getCreateTime,infoRequest.getStartTime(),infoRequest.getEndTime())
                .eq(StringUtils.isNotBlank(infoRequest.getCompanyCode()),TbInfo::getCompanyCode,infoRequest.getCompanyCode())
                .eq(null != infoRequest.getDeviceType(),TbInfo::getDeviceType,infoRequest.getDeviceType())
                .eq(null != infoRequest.getIsNew(),TbInfo::getIsNew,infoRequest.getIsNew())
                .eq(StringUtils.isNotBlank(infoRequest.getDeviceId()),TbInfo::getDeviceId,infoRequest.getDeviceId())
                .eq(StringUtils.isNotBlank(infoRequest.getPointCode()),TbInfo::getPointCode,infoRequest.getPointCode())
                .in(null != infoRequest.getBigDeviceType(),TbInfo::getDeviceType,infoRequest.getDeviceTypes())
                .in(CollectionUtils.isNotEmpty(infoRequest.getPointCodes()),TbInfo::getPointCode,infoRequest.getPointCodes())
                .orderByDesc(TbInfo::getCreateTime);
        page = this.page(page,queryWrapper);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getSize(),page.getCurrent());
    }
}
