package com.usky.dm.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.util.BeanMapperUtils;
import com.usky.common.core.util.StringUtils;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.dm.domain.Company;
import com.usky.dm.domain.DevopsTeam;
import com.usky.dm.domain.DevopsTeamRelation;
import com.usky.dm.domain.WorkOrder;
import com.usky.dm.mapper.WorkOrderMapper;
import com.usky.dm.mapper.vo.WorkOrderRequest;
import com.usky.dm.service.CompanyService;
import com.usky.dm.service.DevopsTeamRelationService;
import com.usky.dm.service.DevopsTeamService;
import com.usky.dm.service.WorkOrderService;
import com.usky.dm.service.util.Arith;
import com.usky.dm.service.util.ExportExcelUtil;
import com.usky.dm.service.vo.AlertCheckVO;
import com.usky.dm.service.vo.WorkOrderExport;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 工单 服务实现类
 * </p>
 *
 * @author ya
 * @since 2022-02-22
 */
@Service
public class WorkOrderServiceImpl extends AbstractCrudService<WorkOrderMapper, WorkOrder> implements WorkOrderService {


    @Autowired
    private DevopsTeamService devopsTeamService;

    @Autowired
    private DevopsTeamRelationService devopsTeamRelationService;

    @Autowired
    private CompanyService companyService;

    @Override
    public boolean add(WorkOrder workOrder) {
        WorkOrder addWork = generateWorkerOrder(wo -> {
            BeanMapperUtils.copy(workOrder, wo);
            wo.setAddPeople(SecurityUtils.getUsername());
            return wo;
        });
        this.save(addWork);
        return true;
    }

    @Override
    public boolean update(WorkOrder workOrder) {
        workOrder.setCzTime(new Date());
        if (StringUtils.isBlank(workOrder.getCzPeople())){
            workOrder.setCzPeople(SecurityUtils.getUsername());
        }
        this.updateById(workOrder);
        return true;
    }

    @Override
    public Boolean checkNameUnique(WorkOrder workOrder) {
        Integer id =  null == workOrder.getId() ? -1 : workOrder.getId();
        LambdaQueryWrapper<WorkOrder> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(WorkOrder::getWorkOrderName,workOrder.getWorkOrderName());
        WorkOrder one = this.getOne(queryWrapper);
        if (Objects.nonNull(one) && one.getId() != id.longValue())
        {
            return true;
        }
        return false;
    }

    @Override
    public CommonPage<WorkOrder> list(WorkOrderRequest workOrderRequest) {
        workOrderRequest.setTeamIds(getTeamIds(workOrderRequest));
        IPage<WorkOrder> page = new Page<>(workOrderRequest.getCurrent(),workOrderRequest.getSize());
        LambdaQueryWrapper<WorkOrder> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper
                .like(StringUtils.isNotBlank(workOrderRequest.getName()),WorkOrder::getWorkOrderName,workOrderRequest.getName())
                .eq(null != workOrderRequest.getStatus(),WorkOrder::getClzt,workOrderRequest.getStatus())
                .between(null != workOrderRequest.getStartTime() && null != workOrderRequest.getEndTime(),WorkOrder::getAddTime,workOrderRequest.getStartTime(),workOrderRequest.getEndTime())
                .eq(null != workOrderRequest.getTeamId(),WorkOrder::getTeamId,workOrderRequest.getTeamId())
                .eq(null != workOrderRequest.getWokeOrderType(),WorkOrder::getWorkOrderType,workOrderRequest.getWokeOrderType())
                .in(CollectionUtils.isNotEmpty(workOrderRequest.getTeamIds()),WorkOrder::getTeamId,workOrderRequest.getTeamIds())
                .in(null != workOrderRequest.getId() && 0 != workOrderRequest.getId(),WorkOrder::getId,workOrderRequest.getId())
                .orderByDesc(WorkOrder::getId);
        page = this.page(page,queryWrapper);
        if (CollectionUtils.isNotEmpty(page.getRecords())){
            enhanceDate(workOrderRequest,page.getRecords());
        }
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getSize(),page.getCurrent());
    }

    /**
     * 获取用户对应的团队id
     * @param workOrderRequest
     * @return
     */
    public List<Integer> getTeamIds(WorkOrderRequest workOrderRequest){
        List<Integer> teamIds = new ArrayList<>();
        if (null == workOrderRequest.getTeamId()){
            if (StringUtils.isNotBlank(workOrderRequest.getUserId()) || StringUtils.isNotBlank(workOrderRequest.getUserName())){
                LambdaQueryWrapper<DevopsTeamRelation> queryWrapper = Wrappers.lambdaQuery();
                queryWrapper
                        .select(DevopsTeamRelation::getFid)
                        .eq(StringUtils.isNotBlank(workOrderRequest.getUserId()),DevopsTeamRelation::getUserId,workOrderRequest.getUserId())
                        .eq(StringUtils.isNotBlank(workOrderRequest.getUserName()),DevopsTeamRelation::getName,workOrderRequest.getUserName())
                        .groupBy(DevopsTeamRelation::getFid);
                List<DevopsTeamRelation> list = devopsTeamRelationService.list(queryWrapper);
                teamIds = list.stream().map(DevopsTeamRelation::getFid).collect(Collectors.toList());
            }
        }
        return teamIds;
    }

    /**
     * 增强数据
     * @param workOrderRequest
     * @param list
     */
    public void enhanceDate(WorkOrderRequest workOrderRequest,List<WorkOrder> list){
        List<DevopsTeam> devopsTeams = new ArrayList<>();
        List<DevopsTeamRelation> devopsTeamRelations = new ArrayList<>();
        if (null != workOrderRequest.getTeamId()){
            devopsTeams.add(devopsTeamService.getById(workOrderRequest.getTeamId()));
            LambdaQueryWrapper<DevopsTeamRelation> queryWrapper = Wrappers.lambdaQuery();
            queryWrapper
                    .eq(DevopsTeamRelation::getFid,workOrderRequest.getTeamId())
                    .eq(DevopsTeamRelation::getLabel,2);
            devopsTeamRelations.add(devopsTeamRelationService.getOne(queryWrapper));
        }
        if (CollectionUtils.isNotEmpty(workOrderRequest.getTeamIds())){
            devopsTeams = getByTeamIds(workOrderRequest.getTeamIds());
            devopsTeamRelations = getTeamUserByTeamIds(workOrderRequest.getTeamIds());
        }
        if (CollectionUtils.isEmpty(workOrderRequest.getTeamIds()) && null == workOrderRequest.getTeamId()){
            List<Integer> teamIds = list.stream()
                    .filter(wo -> null != wo.getTeamId())
                    .collect(
                    Collectors.collectingAndThen(
                            Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(WorkOrder::getTeamId))), ArrayList::new)
            ).stream().map(WorkOrder::getTeamId).collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(teamIds)){
                devopsTeams = getByTeamIds(teamIds);
                devopsTeamRelations = getTeamUserByTeamIds(teamIds);
            }
        }
        List<DevopsTeam> finalDevopsTeams = devopsTeams;
        List<DevopsTeamRelation> finalDevopsTeamRelations = devopsTeamRelations;
        list.forEach(workOrder -> {
            if (null != workOrder.getTeamId()){
                DevopsTeam devopsTeam = finalDevopsTeams.stream().filter(dt -> dt.getId().equals(workOrder.getTeamId())).findAny().get();
                DevopsTeamRelation devopsTeamRelation = finalDevopsTeamRelations.stream().filter(dtr -> dtr.getFid().equals(workOrder.getTeamId())).findAny().get();
                workOrder.setTeamName(devopsTeam.getTeamName());
                workOrder.setDutyName(devopsTeamRelation.getName());
                workOrder.setDutyPhone(devopsTeamRelation.getPhone());
            }
        });
    }




    public List<DevopsTeam> getByTeamIds(List<Integer> teamIds){
        LambdaQueryWrapper<DevopsTeam> teamQueryWrapper = Wrappers.lambdaQuery();
        teamQueryWrapper.in(DevopsTeam::getId,teamIds);
        return devopsTeamService.list(teamQueryWrapper);
    }

    public List<DevopsTeamRelation> getTeamUserByTeamIds(List<Integer> teamIds){
        LambdaQueryWrapper<DevopsTeamRelation> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(DevopsTeamRelation::getFid,teamIds)
                .eq(DevopsTeamRelation::getLabel,2);
        return devopsTeamRelationService.list(queryWrapper);
    }

    @Override
    public AlertCheckVO getCollect() {
        QueryWrapper<WorkOrder> queryWrapper = new QueryWrapper<>();
        queryWrapper.select("clzt","count(*) as count")
                .groupBy("clzt");
        List<Map<String, Object>> list = this.listMaps(queryWrapper);
        AlertCheckVO alertCheckVO = new AlertCheckVO();
        Integer noCheck = list.stream().filter(map -> "0".equals(map.get("clzt").toString())).findAny().map(map -> Integer.parseInt(map.get("count").toString())).orElse(0);
        Integer check = list.stream().filter(map -> "1".equals(map.get("clzt").toString())).findAny().map(map -> Integer.parseInt(map.get("count").toString())).orElse(0);
        Integer check2 = list.stream().filter(map -> "2".equals(map.get("clzt").toString())).findAny().map(map -> Integer.parseInt(map.get("count").toString())).orElse(0);
        alertCheckVO.setCheckCount(check+check2);
        alertCheckVO.setNoCheckCount(noCheck);
        alertCheckVO.setTotal(check+noCheck+check2);
        alertCheckVO.setCheckRadio(0 == alertCheckVO.getTotal() ? 1 : Arith.div(check+check2,alertCheckVO.getTotal()));
        return alertCheckVO;
    }

    @Override
    public String export(WorkOrderRequest workOrderRequest) {
        ExportParams params = new ExportParams(null, "工单");
        return ExportExcelUtil.export("工单", param -> ExcelExportUtil.exportBigExcel(params, WorkOrderExport.class,
                (o, i) -> {
                    workOrderRequest.setCurrent(i);
                    workOrderRequest.setSize(30);
                    CommonPage<WorkOrder> page = this.list(workOrderRequest);
                    return new ArrayList<>(BeanMapperUtils.mapList(page.getRecords(), WorkOrder.class, WorkOrderExport.class));
                }, null));
    }


    @Override
    public boolean remove(Integer id) {
        return false;
    }

    @Override
    public WorkOrder generateWorkerOrder(Function<WorkOrder, WorkOrder> function) {
        WorkOrder workOrder = new WorkOrder();
        workOrder = function.apply(workOrder);
        //添加团队
        if (null == workOrder.getTeamId()){
            LambdaQueryWrapper<DevopsTeam> queryWrapper = Wrappers.lambdaQuery();
            queryWrapper.eq(DevopsTeam::getTeamName,"维修部");
            List<DevopsTeam> list = devopsTeamService.list(queryWrapper);
            if (CollectionUtils.isNotEmpty(list)){
                DevopsTeam devopsTeam = list.get(0);
                workOrder.setTeamId(devopsTeam.getId());
                workOrder.setTeamName(devopsTeam.getTeamName());
            }
        }
        //添加单位
        if (StringUtils.isBlank(workOrder.getCompanyCode())){
            Company company = companyService.getDefault();
            workOrder.setCompanyCode(company.getOwnerId());
            workOrder.setCompanyName(company.getOwnerName());
        }
        return workOrder;
    }

    @Override
    public WorkOrder oneByTypeAndBusiness(Integer type, String business) {
        IPage<WorkOrder> page = new Page<>(1,1);
        LambdaQueryWrapper<WorkOrder> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(WorkOrder::getWorkOrderType,type)
                .eq(WorkOrder::getBusinessId,business)
                .orderByDesc(WorkOrder::getAddTime);
        page = this.page(page,queryWrapper);
        if (CollectionUtils.isNotEmpty(page.getRecords())){
            Integer teamId = page.getRecords().get(0).getTeamId();
            if (null != teamId){
                WorkOrderRequest workOrderRequest = new WorkOrderRequest();
                workOrderRequest.setTeamId(teamId);
                enhanceDate(workOrderRequest,page.getRecords());
            }
            return page.getRecords().get(0);
        }
        return null;
    }

    @Override
    public Boolean checkWorkOrder(WorkOrder workOrder, Function<WorkOrder, Boolean> function) {
        if (null == workOrder.getId()){
            workOrder = this.oneByTypeAndBusiness(workOrder.getWorkOrderType(), workOrder.getBusinessId());
        }
        workOrder.setClzt(1);
        workOrder.setCzTime(new Date());
        if (StringUtils.isBlank(workOrder.getCzPeople())){
            workOrder.setCzPeople(SecurityUtils.getUsername());
        }
        this.updateById(workOrder);
        return function.apply(workOrder);
    }
}
