package com.usky.dm.service.util;

import com.usky.common.core.exception.BusinessException;
import com.usky.common.core.util.FileUtils;
import com.usky.common.core.util.GlobalUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.function.Function;

/**
 * 通用的导出表格
 * @author yq
 * @date 2022/2/24 9:58
 */
@Slf4j
public class ExportExcelUtil {

    public static String export(String name,Function<Object,Workbook> function) {
        Workbook workbook = null;
        File file = null;
        try {
            workbook = function.apply(name);
            if (null != workbook) {
                file = FileUtils.getFile(GlobalUtils.getTempBaseDir(), String.format("%s-%s.xlsx", name, System.currentTimeMillis() + ""));
                FileUtils.createFile(file.getAbsolutePath());
                FileOutputStream allListingFileOutputStream = new FileOutputStream(file);
                workbook.write(allListingFileOutputStream);
            } else {
                throw new BusinessException("表格数据为空");
            }
        } catch (Exception e) {
            log.error("导出文件失败", e);
            throw new BusinessException("导出文件失败");
        } finally {
            if (workbook != null) {
                try {
                    workbook.close();
                } catch (IOException e) {
                    log.error("===export spec=== 关闭workbook失败", e);
                }
            }
        }
        return file.getName();
    }
}
